from typing import overload
from enum import Enum
import abc
import datetime
import typing
import warnings

import System
import System.Collections
import System.Globalization
import System.Reflection
import System.Runtime.Serialization
import System.Text

System_Globalization_SortVersion = typing.Any


class SortKey(System.Object):
    """Represents the result of mapping a string to its sort key."""

    @property
    def original_string(self) -> str:
        """
        Returns the original string used to create the current instance
        of SortKey.
        """
        ...

    @property
    def key_data(self) -> typing.List[int]:
        """
        Returns a byte array representing the current instance of the
        sort key.
        """
        ...

    @staticmethod
    def compare(sortkey_1: System.Globalization.SortKey, sortkey_2: System.Globalization.SortKey) -> int:
        """
        Compares the two sort keys.  Returns 0 if the two sort keys are
        equal, a number less than 0 if sortkey_1 is less than sortkey_2,
        and a number greater than 0 if sortkey_1 is greater than sortkey_2.
        """
        ...

    def equals(self, value: typing.Any) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    def to_string(self) -> str:
        ...


class IdnMapping(System.Object):
    """This class has no documentation."""

    @property
    def allow_unassigned(self) -> bool:
        ...

    @property.setter
    def allow_unassigned(self, value: bool) -> None:
        ...

    @property
    def use_std_3_ascii_rules(self) -> bool:
        ...

    @property.setter
    def use_std_3_ascii_rules(self, value: bool) -> None:
        ...

    def __init__(self) -> None:
        ...

    def equals(self, obj: typing.Any) -> bool:
        ...

    @overload
    def get_ascii(self, unicode: str) -> str:
        ...

    @overload
    def get_ascii(self, unicode: str, index: int) -> str:
        ...

    @overload
    def get_ascii(self, unicode: str, index: int, count: int) -> str:
        ...

    def get_hash_code(self) -> int:
        ...

    @overload
    def get_unicode(self, ascii: str) -> str:
        ...

    @overload
    def get_unicode(self, ascii: str, index: int) -> str:
        ...

    @overload
    def get_unicode(self, ascii: str, index: int, count: int) -> str:
        ...


class SortVersion(System.Object, System.IEquatable[System_Globalization_SortVersion]):
    """This class has no documentation."""

    @property
    def full_version(self) -> int:
        ...

    @property
    def sort_id(self) -> System.Guid:
        ...

    def __init__(self, fullVersion: int, sortId: System.Guid) -> None:
        ...

    @overload
    def equals(self, obj: typing.Any) -> bool:
        ...

    @overload
    def equals(self, other: System.Globalization.SortVersion) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...


class CompareOptions(Enum):
    """Defines the string comparison options to use with CompareInfo."""

    NONE = ...
    """Indicates the default option settings for string comparisons"""

    IGNORE_CASE = ...
    """Indicates that the string comparison must ignore case."""

    IGNORE_NON_SPACE = ...
    """
    Indicates that the string comparison must ignore nonspacing combining characters, such as diacritics.
    The https://go.microsoft.com/fwlink/?linkid=37123 defines combining characters as
    characters that are combined with base characters to produce a new character. Nonspacing combining characters do not
    occupy a spacing position by themselves when rendered.
    """

    IGNORE_SYMBOLS = ...
    """
    Indicates that the string comparison must ignore symbols, such as white-space characters, punctuation, currency symbols,
    the percent sign, mathematical symbols, the ampersand, and so on.
    """

    IGNORE_KANA_TYPE = ...
    """
    Indicates that the string comparison must ignore the Kana type. Kana type refers to Japanese hiragana and katakana characters, which represent phonetic sounds in the Japanese language.
    Hiragana is used for native Japanese expressions and words, while katakana is used for words borrowed from other languages, such as "computer" or "Internet".
    A phonetic sound can be expressed in both hiragana and katakana. If this value is selected, the hiragana character for one sound is considered equal to the katakana character for the same sound.
    """

    IGNORE_WIDTH = ...
    """
    Indicates that the string comparison must ignore the character width. For example, Japanese katakana characters can be written as full-width or half-width.
    If this value is selected, the katakana characters written as full-width are considered equal to the same characters written as half-width.
    """

    NUMERIC_ORDERING = ...
    """
    Indicates that the string comparison must sort sequences of digits (Unicode general category "Nd") based on their numeric value.
    For example, "2" comes before "10". Non-digit characters such as decimal points, minus or plus signs, etc.
    are not considered as part of the sequence and will terminate it. This flag is not valid for indexing
    (such as CompareInfo.IndexOf(string, string, CompareOptions), CompareInfo.IsPrefix(string, string, CompareOptions), etc.).
    """

    ORDINAL_IGNORE_CASE = ...
    """
    String comparison must ignore case, then perform an ordinal comparison. This technique is equivalent to
    converting the string to uppercase using the invariant culture and then performing an ordinal comparison on the result.
    This value cannot be combined with other CompareOptions values and must be used alone.
    """

    STRING_SORT = ...
    """
    Indicates that the string comparison must use the string sort algorithm. In a string sort, the hyphen and the apostrophe,
    as well as other nonalphanumeric symbols, come before alphanumeric characters.
    """

    ORDINAL = ...
    """
    Indicates that the string comparison must use successive Unicode UTF-16 encoded values of the string (code unit by code unit comparison),
    leading to a fast comparison but one that is culture-insensitive. A string starting with a code unit XXXX16 comes before a string starting with YYYY16,
    if XXXX16 is less than YYYY16. This value cannot be combined with other CompareOptions values and must be used alone.
    """


class CompareInfo(System.Object, System.Runtime.Serialization.IDeserializationCallback):
    """This class implements a set of methods for comparing strings."""

    @property
    def name(self) -> str:
        """
        Returns the name of the culture (well actually, of the sort).
         Very important for providing a non-LCID way of identifying
         what the sort is.
        
         Note that this name isn't dereferenced in case the CompareInfo is a different locale
         which is consistent with the behaviors of earlier versions.  (so if you ask for a sort
         and the locale's changed behavior, then you'll get changed behavior, which is like
         what happens for a version update)
        """
        ...

    @property
    def version(self) -> System.Globalization.SortVersion:
        ...

    @property
    def lcid(self) -> int:
        ...

    @overload
    def compare(self, string_1: str, string_2: str) -> int:
        """
        Compares the two strings with the given options.  Returns 0 if the
        two strings are equal, a number less than 0 if string_1 is less
        than string_2, and a number greater than 0 if string_1 is greater
        than string_2.
        """
        ...

    @overload
    def compare(self, string_1: str, string_2: str, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def compare(self, string_1: str, offset_1: int, length_1: int, string_2: str, offset_2: int, length_2: int) -> int:
        """
        Compares the specified regions of the two strings with the given
        options.
        Returns 0 if the two strings are equal, a number less than 0 if
        string_1 is less than string_2, and a number greater than 0 if
        string_1 is greater than string_2.
        """
        ...

    @overload
    def compare(self, string_1: str, offset_1: int, string_2: str, offset_2: int, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def compare(self, string_1: str, offset_1: int, string_2: str, offset_2: int) -> int:
        ...

    @overload
    def compare(self, string_1: str, offset_1: int, length_1: int, string_2: str, offset_2: int, length_2: int, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def compare(self, string_1: System.ReadOnlySpan[str], string_2: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions = ...) -> int:
        """
        Compares two strings.
        
        :param string_1: The first string to compare.
        :param string_2: The second string to compare.
        :param options: The CompareOptions to use during the comparison.
        :returns: Zero if  and  are equal; or a negative value if  sorts before ; or a positive value if  sorts after .
        """
        ...

    def equals(self, value: typing.Any) -> bool:
        ...

    @staticmethod
    @overload
    def get_compare_info(culture: int, assembly: System.Reflection.Assembly) -> System.Globalization.CompareInfo:
        """
        Get the CompareInfo constructed from the data table in the specified
        assembly for the specified culture.
        Warning: The assembly versioning mechanism is dead!
        """
        ...

    @staticmethod
    @overload
    def get_compare_info(name: str, assembly: System.Reflection.Assembly) -> System.Globalization.CompareInfo:
        """
        Get the CompareInfo constructed from the data table in the specified
        assembly for the specified culture.
        The purpose of this method is to provide version for CompareInfo tables.
        """
        ...

    @staticmethod
    @overload
    def get_compare_info(culture: int) -> System.Globalization.CompareInfo:
        """
        Get the CompareInfo for the specified culture.
        This method is provided for ease of integration with NLS-based software.
        """
        ...

    @staticmethod
    @overload
    def get_compare_info(name: str) -> System.Globalization.CompareInfo:
        """Get the CompareInfo for the specified culture."""
        ...

    @overload
    def get_hash_code(self) -> int:
        ...

    @overload
    def get_hash_code(self, source: str, options: System.Globalization.CompareOptions) -> int:
        """
        This method performs the equivalent of of creating a Sortkey for a string from CompareInfo,
        then generates a randomized hashcode value from the sort key.
        
        The hash code is guaranteed to be the same for string A and B where A.Equals(B) is true and both
        the CompareInfo and the CompareOptions are the same. If two different CompareInfo objects
        treat the string the same way, this implementation will treat them differently (the same way that
        Sortkey does at the moment).
        """
        ...

    @overload
    def get_hash_code(self, source: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def get_sort_key(self, source: str, options: System.Globalization.CompareOptions) -> System.Globalization.SortKey:
        """Gets the SortKey for the given string with the given options."""
        ...

    @overload
    def get_sort_key(self, source: str) -> System.Globalization.SortKey:
        ...

    @overload
    def get_sort_key(self, source: System.ReadOnlySpan[str], destination: System.Span[int], options: System.Globalization.CompareOptions = ...) -> int:
        """
        Computes a sort key over the specified input.
        
        :param source: The text over which to compute the sort key.
        :param destination: The buffer into which to write the resulting sort key bytes.
        :param options: The CompareOptions used for computing the sort key.
        :returns: The number of bytes written to .
        """
        ...

    def get_sort_key_length(self, source: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions = ...) -> int:
        """
        Returns the length (in bytes) of the sort key that would be produced from the specified input.
        
        :param source: The text over which to compute the sort key.
        :param options: The CompareOptions used for computing the sort key.
        :returns: The length (in bytes) of the sort key.
        """
        ...

    @overload
    def index_of(self, source: str, value: str) -> int:
        """
        Returns the first index where value is found in string.  The
        search starts from startIndex and ends at endIndex.  Returns -1 if
        the specified value is not found.  If value equals string.Empty,
        startIndex is returned.  Throws IndexOutOfRange if startIndex or
        endIndex is less than zero or greater than the length of string.
        Throws ArgumentException if value (as a string) is null.
        """
        ...

    @overload
    def index_of(self, source: str, value: str) -> int:
        ...

    @overload
    def index_of(self, source: str, value: str, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def index_of(self, source: str, value: str, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def index_of(self, source: str, value: str, start_index: int) -> int:
        ...

    @overload
    def index_of(self, source: str, value: str, start_index: int) -> int:
        ...

    @overload
    def index_of(self, source: str, value: str, start_index: int, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def index_of(self, source: str, value: str, start_index: int, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def index_of(self, source: str, value: str, start_index: int, count: int) -> int:
        ...

    @overload
    def index_of(self, source: str, value: str, start_index: int, count: int) -> int:
        ...

    @overload
    def index_of(self, source: str, value: str, start_index: int, count: int, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def index_of(self, source: str, value: str, start_index: int, count: int, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def index_of(self, source: System.ReadOnlySpan[str], value: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions = ...) -> int:
        """
        Searches for the first occurrence of a substring within a source string.
        
        :param source: The string to search within.
        :param value: The substring to locate within .
        :param options: The CompareOptions to use during the search.
        :returns: The zero-based index into  where the substring  first appears; or -1 if  cannot be found within .
        """
        ...

    @overload
    def index_of(self, source: System.ReadOnlySpan[str], value: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions, match_length: typing.Optional[int]) -> typing.Union[int, int]:
        """
        Searches for the first occurrence of a substring within a source string.
        
        :param source: The string to search within.
        :param value: The substring to locate within .
        :param options: The CompareOptions to use during the search.
        :param match_length: When this method returns, contains the number of characters of  that matched the desired value. This may be different than the length of  if a linguistic comparison is performed. Set to 0 if  is not found within .
        :returns: The zero-based index into  where the substring  first appears; or -1 if  cannot be found within .
        """
        ...

    @overload
    def index_of(self, source: System.ReadOnlySpan[str], value: System.Text.Rune, options: System.Globalization.CompareOptions = ...) -> int:
        """
        Searches for the first occurrence of a Rune within a source string.
        
        :param source: The string to search within.
        :param value: The Rune to locate within .
        :param options: The CompareOptions to use during the search.
        :returns: The zero-based index into  where  first appears; or -1 if  cannot be found within .
        """
        ...

    @overload
    def is_prefix(self, source: str, prefix: str, options: System.Globalization.CompareOptions) -> bool:
        """
        Determines whether prefix is a prefix of string.  If prefix equals
        string.Empty, true is returned.
        """
        ...

    @overload
    def is_prefix(self, source: System.ReadOnlySpan[str], prefix: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions = ...) -> bool:
        """
        Determines whether a string starts with a specific prefix.
        
        :param source: The string to search within.
        :param prefix: The prefix to attempt to match at the start of .
        :param options: The CompareOptions to use during the match.
        :returns: true if  occurs at the start of ; otherwise, false.
        """
        ...

    @overload
    def is_prefix(self, source: System.ReadOnlySpan[str], prefix: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions, match_length: typing.Optional[int]) -> typing.Union[bool, int]:
        """
        Determines whether a string starts with a specific prefix.
        
        :param source: The string to search within.
        :param prefix: The prefix to attempt to match at the start of .
        :param options: The CompareOptions to use during the match.
        :param match_length: When this method returns, contains the number of characters of  that matched the desired prefix. This may be different than the length of  if a linguistic comparison is performed. Set to 0 if the prefix did not match.
        :returns: true if  occurs at the start of ; otherwise, false.
        """
        ...

    @overload
    def is_prefix(self, source: str, prefix: str) -> bool:
        ...

    @staticmethod
    @overload
    def is_sortable(ch: str) -> bool:
        ...

    @staticmethod
    @overload
    def is_sortable(text: str) -> bool:
        ...

    @staticmethod
    @overload
    def is_sortable(text: System.ReadOnlySpan[str]) -> bool:
        """
        Indicates whether a specified Unicode string is sortable.
        
        :param text: A string of zero or more Unicode characters.
        :returns: true if  is non-empty and contains only sortable Unicode characters; otherwise, false.
        """
        ...

    @staticmethod
    @overload
    def is_sortable(value: System.Text.Rune) -> bool:
        """
        Indicates whether a specified Rune is sortable.
        
        :param value: A Unicode scalar value.
        :returns: true if  is a sortable Unicode scalar value; otherwise, false.
        """
        ...

    @overload
    def is_suffix(self, source: str, suffix: str, options: System.Globalization.CompareOptions) -> bool:
        """
        Determines whether suffix is a suffix of string.  If suffix equals
        string.Empty, true is returned.
        """
        ...

    @overload
    def is_suffix(self, source: System.ReadOnlySpan[str], suffix: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions = ...) -> bool:
        """
        Determines whether a string ends with a specific suffix.
        
        :param source: The string to search within.
        :param suffix: The suffix to attempt to match at the end of .
        :param options: The CompareOptions to use during the match.
        :returns: true if  occurs at the end of ; otherwise, false.
        """
        ...

    @overload
    def is_suffix(self, source: System.ReadOnlySpan[str], suffix: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions, match_length: typing.Optional[int]) -> typing.Union[bool, int]:
        """
        Determines whether a string ends with a specific suffix.
        
        :param source: The string to search within.
        :param suffix: The suffix to attempt to match at the end of .
        :param options: The CompareOptions to use during the match.
        :param match_length: When this method returns, contains the number of characters of  that matched the desired suffix. This may be different than the length of  if a linguistic comparison is performed. Set to 0 if the suffix did not match.
        :returns: true if  occurs at the end of ; otherwise, false.
        """
        ...

    @overload
    def is_suffix(self, source: str, suffix: str) -> bool:
        ...

    @overload
    def last_index_of(self, source: str, value: str) -> int:
        """
        Returns the last index where value is found in string.  The
        search starts from startIndex and ends at endIndex.  Returns -1 if
        the specified value is not found.  If value equals string.Empty,
        endIndex is returned.  Throws IndexOutOfRange if startIndex or
        endIndex is less than zero or greater than the length of string.
        Throws ArgumentException if value (as a string) is null.
        """
        ...

    @overload
    def last_index_of(self, source: str, value: str) -> int:
        ...

    @overload
    def last_index_of(self, source: str, value: str, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def last_index_of(self, source: str, value: str, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def last_index_of(self, source: str, value: str, start_index: int) -> int:
        ...

    @overload
    def last_index_of(self, source: str, value: str, start_index: int) -> int:
        ...

    @overload
    def last_index_of(self, source: str, value: str, start_index: int, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def last_index_of(self, source: str, value: str, start_index: int, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def last_index_of(self, source: str, value: str, start_index: int, count: int) -> int:
        ...

    @overload
    def last_index_of(self, source: str, value: str, start_index: int, count: int) -> int:
        ...

    @overload
    def last_index_of(self, source: str, value: str, start_index: int, count: int, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def last_index_of(self, source: str, value: str, start_index: int, count: int, options: System.Globalization.CompareOptions) -> int:
        ...

    @overload
    def last_index_of(self, source: System.ReadOnlySpan[str], value: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions = ...) -> int:
        """
        Searches for the last occurrence of a substring within a source string.
        
        :param source: The string to search within.
        :param value: The substring to locate within .
        :param options: The CompareOptions to use during the search.
        :returns: The zero-based index into  where the substring  last appears; or -1 if  cannot be found within .
        """
        ...

    @overload
    def last_index_of(self, source: System.ReadOnlySpan[str], value: System.ReadOnlySpan[str], options: System.Globalization.CompareOptions, match_length: typing.Optional[int]) -> typing.Union[int, int]:
        """
        Searches for the last occurrence of a substring within a source string.
        
        :param source: The string to search within.
        :param value: The substring to locate within .
        :param options: The CompareOptions to use during the search.
        :param match_length: When this method returns, contains the number of characters of  that matched the desired value. This may be different than the length of  if a linguistic comparison is performed. Set to 0 if  is not found within .
        :returns: The zero-based index into  where the substring  last appears; or -1 if  cannot be found within .
        """
        ...

    @overload
    def last_index_of(self, source: System.ReadOnlySpan[str], value: System.Text.Rune, options: System.Globalization.CompareOptions = ...) -> int:
        """
        Searches for the last occurrence of a Rune within a source string.
        
        :param source: The string to search within.
        :param value: The Rune to locate within .
        :param options: The CompareOptions to use during the search.
        :returns: The zero-based index into  where  last appears; or -1 if  cannot be found within .
        """
        ...

    def to_string(self) -> str:
        ...


class DaylightTime(System.Object):
    """This class has no documentation."""

    @property
    def start(self) -> datetime.datetime:
        ...

    @property
    def end(self) -> datetime.datetime:
        ...

    @property
    def delta(self) -> datetime.timedelta:
        ...

    def __init__(self, start: typing.Union[datetime.datetime, datetime.date], end: typing.Union[datetime.datetime, datetime.date], delta: datetime.timedelta) -> None:
        ...


class CalendarAlgorithmType(Enum):
    """This class has no documentation."""

    UNKNOWN = 0

    SOLAR_CALENDAR = 1

    LUNAR_CALENDAR = 2

    LUNISOLAR_CALENDAR = 3


class CalendarWeekRule(Enum):
    """This class has no documentation."""

    FIRST_DAY = 0

    FIRST_FULL_WEEK = 1

    FIRST_FOUR_DAY_WEEK = 2


class Calendar(System.Object, System.ICloneable, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def is_read_only(self) -> bool:
        ...

    CURRENT_ERA: int = 0

    @property
    @abc.abstractmethod
    def eras(self) -> typing.List[int]:
        """Get the list of era values."""
        ...

    @property
    def days_in_year_before_min_supported_year(self) -> int:
        """This property is protected."""
        ...

    @property
    def two_digit_year_max(self) -> int:
        """
        Returns and assigns the maximum value to represent a two digit year.
        This value is the upper boundary of a 100 year range that allows a
        two digit year to be properly translated to a four digit year.
        For example, if 2049 is the upper boundary, then a two digit value of
        30 should be interpreted as 1950 while a two digit value of 49 should
        be interpreted as 2049.  In this example, the 100 year range would be
        from 1950-2049.  See ToFourDigitYear().
        """
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def __init__(self) -> None:
        """This method is protected."""
        ...

    def add_days(self, time: typing.Union[datetime.datetime, datetime.date], days: int) -> datetime.datetime:
        """
        Returns the DateTime resulting from adding a fractional number of
        days to the specified DateTime. The result is computed by rounding the
        fractional number of days given by value to the nearest
        millisecond, and adding that interval to the specified DateTime. The
        value argument is permitted to be negative.
        """
        ...

    def add_hours(self, time: typing.Union[datetime.datetime, datetime.date], hours: int) -> datetime.datetime:
        """
        Returns the DateTime resulting from adding a fractional number of
        hours to the specified DateTime. The result is computed by rounding the
        fractional number of hours given by value to the nearest
        millisecond, and adding that interval to the specified DateTime. The
        value argument is permitted to be negative.
        """
        ...

    def add_milliseconds(self, time: typing.Union[datetime.datetime, datetime.date], milliseconds: float) -> datetime.datetime:
        """
        Returns the DateTime resulting from adding the given number of
        milliseconds to the specified DateTime. The result is computed by rounding
        the number of milliseconds given by value to the nearest integer,
        and adding that interval to the specified DateTime. The value
        argument is permitted to be negative.
        """
        ...

    def add_minutes(self, time: typing.Union[datetime.datetime, datetime.date], minutes: int) -> datetime.datetime:
        """
        Returns the DateTime resulting from adding a fractional number of
        minutes to the specified DateTime. The result is computed by rounding the
        fractional number of minutes given by value to the nearest
        millisecond, and adding that interval to the specified DateTime. The
        value argument is permitted to be negative.
        """
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        """
        Returns the DateTime resulting from adding the given number of
        months to the specified DateTime. The result is computed by incrementing
        (or decrementing) the year and month parts of the specified DateTime by
        value months, and, if required, adjusting the day part of the
        resulting date downwards to the last day of the resulting month in the
        resulting year. The time-of-day part of the result is the same as the
        time-of-day part of the specified DateTime.
        
        In more precise terms, considering the specified DateTime to be of the
        form y / m / d + t, where y is the
        year, m is the month, d is the day, and t is the
        time-of-day, the result is y1 / m1 / d1 + t,
        where y1 and m1 are computed by adding value months
        to y and m, and d1 is the largest value less than
        or equal to d that denotes a valid day in month m1 of year
        y1.
        """
        ...

    def add_seconds(self, time: typing.Union[datetime.datetime, datetime.date], seconds: int) -> datetime.datetime:
        """
        Returns the DateTime resulting from adding a number of
        seconds to the specified DateTime. The result is computed by rounding the
        fractional number of seconds given by value to the nearest
        millisecond, and adding that interval to the specified DateTime. The
        value argument is permitted to be negative.
        """
        ...

    def add_weeks(self, time: typing.Union[datetime.datetime, datetime.date], weeks: int) -> datetime.datetime:
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        """
        Returns the DateTime resulting from adding the given number of
        years to the specified DateTime. The result is computed by incrementing
        (or decrementing) the year part of the specified DateTime by value
        years. If the month and day of the specified DateTime is 2/29, and if the
        resulting year is not a leap year, the month and day of the resulting
        DateTime becomes 2/28. Otherwise, the month, day, and time-of-day
        parts of the result are the same as those of the specified DateTime.
        """
        ...

    def clone(self) -> System.Object:
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the day-of-month part of the specified DateTime. The returned
        value is an integer between 1 and 31.
        """
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        """
        Returns the day-of-week part of the specified DateTime. The returned value
        is an integer between 0 and 6, where 0 indicates Sunday, 1 indicates
        Monday, 2 indicates Tuesday, 3 indicates Wednesday, 4 indicates
        Thursday, 5 indicates Friday, and 6 indicates Saturday.
        """
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the day-of-year part of the specified DateTime. The returned value
        is an integer between 1 and 366.
        """
        ...

    @overload
    def get_days_in_month(self, year: int, month: int) -> int:
        """
        Returns the number of days in the month given by the year and
        month arguments.
        """
        ...

    @overload
    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        """
        Returns the number of days in the month given by the year and
        month arguments for the specified era.
        """
        ...

    @overload
    def get_days_in_year(self, year: int) -> int:
        """
        Returns the number of days in the year given by the year argument
        for the current era.
        """
        ...

    @overload
    def get_days_in_year(self, year: int, era: int) -> int:
        """
        Returns the number of days in the year given by the year argument
        for the current era.
        """
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """Returns the era for the specified DateTime value."""
        ...

    def get_hour(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    @overload
    def get_leap_month(self, year: int) -> int:
        """
        Returns  the leap month in a calendar year of the current era.
        This method returns 0 if this calendar does not have leap month,
        or this year is not a leap year.
        """
        ...

    @overload
    def get_leap_month(self, year: int, era: int) -> int:
        """
        Returns  the leap month in a calendar year of the specified era.
        This method returns 0 if this calendar does not have leap month,
        or this year is not a leap year.
        """
        ...

    def get_milliseconds(self, time: typing.Union[datetime.datetime, datetime.date]) -> float:
        ...

    def get_minute(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    @overload
    def get_months_in_year(self, year: int) -> int:
        ...

    @overload
    def get_months_in_year(self, year: int, era: int) -> int:
        ...

    def get_second(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_week_of_year(self, time: typing.Union[datetime.datetime, datetime.date], rule: System.Globalization.CalendarWeekRule, first_day_of_week: System.DayOfWeek) -> int:
        """
        Returns the week of year for the specified DateTime. The returned value is an
        integer between 1 and 53.
        """
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the year part of the specified DateTime. The returned value is an
        integer between 1 and 9999.
        """
        ...

    @overload
    def is_leap_day(self, year: int, month: int, day: int) -> bool:
        """
        Checks whether a given day in the current era is a leap day.
        This method returns true if the date is a leap day, or false if not.
        """
        ...

    @overload
    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        """
        Checks whether a given day in the specified era is a leap day.
        This method returns true if the date is a leap day, or false if not.
        """
        ...

    @overload
    def is_leap_month(self, year: int, month: int) -> bool:
        """
        Checks whether a given month in the current era is a leap month.
        This method returns true if month is a leap month, or false if not.
        """
        ...

    @overload
    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        """
        Checks whether a given month in the specified era is a leap month. This method returns true if
        month is a leap month, or false if not.
        """
        ...

    @overload
    def is_leap_year(self, year: int) -> bool:
        """
        Checks whether a given year in the current era is a leap year.
        This method returns true if year is a leap year, or false if not.
        """
        ...

    @overload
    def is_leap_year(self, year: int, era: int) -> bool:
        """
        Checks whether a given year in the specified era is a leap year.
        This method returns true if year is a leap year, or false if not.
        """
        ...

    @staticmethod
    def read_only(calendar: System.Globalization.Calendar) -> System.Globalization.Calendar:
        ...

    @overload
    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int) -> datetime.datetime:
        """
        Returns the date and time converted to a DateTime value.
        Throws an exception if the n-tuple is invalid.
        """
        ...

    @overload
    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        """
        Returns the date and time converted to a DateTime value.
        Throws an exception if the n-tuple is invalid.
        """
        ...

    def to_four_digit_year(self, year: int) -> int:
        """
        Converts the year value to the appropriate century by using the
        TwoDigitYearMax property.  For example, if the TwoDigitYearMax value is 2049,
        then a two digit value of 50 will get converted to 1950 while a two digit
        value of 49 will get converted to 2049.
        """
        ...


class GregorianCalendarTypes(Enum):
    """This class has no documentation."""

    LOCALIZED = ...

    US_ENGLISH = ...

    MIDDLE_EAST_FRENCH = ...

    ARABIC = ...

    TRANSLITERATED_ENGLISH = ...

    TRANSLITERATED_FRENCH = ...


class GregorianCalendar(System.Globalization.Calendar):
    """This class has no documentation."""

    AD_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def calendar_type(self) -> System.Globalization.GregorianCalendarTypes:
        ...

    @property.setter
    def calendar_type(self, value: System.Globalization.GregorianCalendarTypes) -> None:
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, type: System.Globalization.GregorianCalendarTypes) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        """
        Returns the DateTime resulting from adding the given number of
        months to the specified DateTime. The result is computed by incrementing
        (or decrementing) the year and month parts of the specified DateTime by
        value months, and, if required, adjusting the day part of the
        resulting date downwards to the last day of the resulting month in the
        resulting year. The time-of-day part of the result is the same as the
        time-of-day part of the specified DateTime.
        
        In more precise terms, considering the specified DateTime to be of the
        form y / m / d + t, where y is the
        year, m is the month, d is the day, and t is the
        time-of-day, the result is y1 / m1 / d1 + t,
        where y1 and m1 are computed by adding value months
        to y and m, and d1 is the largest value less than
        or equal to d that denotes a valid day in month m1 of year
        y1.
        """
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        """
        Returns the DateTime resulting from adding the given number of
        years to the specified DateTime. The result is computed by incrementing
        (or decrementing) the year part of the specified DateTime by value
        years. If the month and day of the specified DateTime is 2/29, and if the
        resulting year is not a leap year, the month and day of the resulting
        DateTime becomes 2/28. Otherwise, the month, day, and time-of-day
        parts of the result are the same as those of the specified DateTime.
        """
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the day-of-month part of the specified DateTime. The returned
        value is an integer between 1 and 31.
        """
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        """
        Returns the day-of-week part of the specified DateTime. The returned value
        is an integer between 0 and 6, where 0 indicates Sunday, 1 indicates
        Monday, 2 indicates Tuesday, 3 indicates Wednesday, 4 indicates
        Thursday, 5 indicates Friday, and 6 indicates Saturday.
        """
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the day-of-year part of the specified DateTime. The returned value
        is an integer between 1 and 366.
        """
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        """
        Returns the number of days in the month given by the year and
        month arguments.
        """
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        """
        Returns the number of days in the year given by the year argument for
        the current era.
        """
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        """
        Returns the leap month in a calendar year of the specified era.
        This method returns 0 if this calendar does not have leap month, or
        this year is not a leap year.
        """
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the month part of the specified DateTime.
        The returned value is an integer between 1 and 12.
        """
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        """Returns the number of months in the specified year and era."""
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the year part of the specified DateTime. The returned value is an
        integer between 1 and 9999.
        """
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        """
        Checks whether a given day in the specified era is a leap day. This method returns true if
        the date is a leap day, or false if not.
        """
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        """
        Checks whether a given month in the specified era is a leap month.
        This method returns true if month is a leap month, or false if not.
        """
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        """
        Checks whether a given year in the specified era is a leap year. This method returns true if
        year is a leap year, or false if not.
        """
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        """
        Returns the date and time converted to a DateTime value.
        Throws an exception if the n-tuple is invalid.
        """
        ...

    def to_four_digit_year(self, year: int) -> int:
        ...


class KoreanCalendar(System.Globalization.Calendar):
    """
    Korean calendar is based on the Gregorian calendar.  And the year is an offset to Gregorian calendar.
    That is,
         Korean year = Gregorian year + 2333.  So 2000/01/01 A.D. is Korean 4333/01/01
    
    0001/1/1 A.D. is Korean year 2334.
    """

    KOREAN_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def __init__(self) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        ...

    def get_week_of_year(self, time: typing.Union[datetime.datetime, datetime.date], rule: System.Globalization.CalendarWeekRule, first_day_of_week: System.DayOfWeek) -> int:
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        ...

    def to_four_digit_year(self, year: int) -> int:
        ...


class EastAsianLunisolarCalendar(System.Globalization.Calendar, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        """
        Returns the DateTime resulting from adding the given number of
        months to the specified DateTime. The result is computed by incrementing
        (or decrementing) the year and month parts of the specified DateTime by
        value months, and, if required, adjusting the day part of the
        resulting date downwards to the last day of the resulting month in the
        resulting year. The time-of-day part of the result is the same as the
        time-of-day part of the specified DateTime.
        """
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        ...

    def get_celestial_stem(self, sexagenary_year: int) -> int:
        """
        Return the celestial year from the 60-year cycle.
        The returned value is from 1 ~ 10.
        """
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the day-of-month part of the specified DateTime. The returned
        value is an integer between 1 and 29 or 30.
        """
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        """
        Returns the day-of-week part of the specified DateTime. The returned value
        is an integer between 0 and 6, where 0 indicates Sunday, 1 indicates
        Monday, 2 indicates Tuesday, 3 indicates Wednesday, 4 indicates
        Thursday, 5 indicates Friday, and 6 indicates Saturday.
        """
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the day-of-year part of the specified DateTime. The returned value
        is an integer between 1 and [354|355 |383|384].
        """
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        """
        Returns the number of days in the month given by the year and
        month arguments.
        """
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        """Returns the number of days in the year given by the year argument for the current era."""
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        """
        Returns  the leap month in a calendar year of the specified era. This method returns 0
        if this year is not a leap year.
        """
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the month part of the specified DateTime.
        The returned value is an integer between 1 and 13.
        """
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        """Returns the number of months in the specified year and era."""
        ...

    def get_sexagenary_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """Return the year number in the 60-year cycle."""
        ...

    def get_terrestrial_branch(self, sexagenary_year: int) -> int:
        """
        Return the Terrestial Branch from the 60-year cycle.
        The returned value is from 1 ~ 12.
        """
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        """
        Returns the year part of the specified DateTime.
        The returned value is an integer between 1 and MaxCalendarYear.
        """
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        """
        Checks whether a given day in the specified era is a leap day.
        This method returns true if the date is a leap day, or false if not.
        """
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        """
        Checks whether a given month in the specified era is a leap month.
        This method returns true if month is a leap month, or false if not.
        """
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        """
        Checks whether a given year in the specified era is a leap year.
        This method returns true if year is a leap year, or false if not.
        """
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        """
        Returns the date and time converted to a DateTime value.
        Throws an exception if the n-tuple is invalid.
        """
        ...

    def to_four_digit_year(self, year: int) -> int:
        ...


class TaiwanLunisolarCalendar(System.Globalization.EastAsianLunisolarCalendar):
    """This class has no documentation."""

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def days_in_year_before_min_supported_year(self) -> int:
        """This property is protected."""
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    def __init__(self) -> None:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...


class JapaneseCalendar(System.Globalization.Calendar):
    """
    JapaneseCalendar is based on Gregorian calendar.  The month and day values are the same as
    Gregorian calendar. However, the year value is an offset to the Gregorian
    year based on the era.
    
    This system is adopted by Emperor Meiji in 1868. The year value is counted based on the reign of an emperor,
    and the era begins on the day an emperor ascends the throne and continues until his death.
    The era changes at 12:00AM.
    
    For example, the current era is Reiwa. It started on 2019/5/1 A.D.  Therefore, Gregorian year 2019 is also Reiwa 1st.
    2019/5/1 A.D. is also Reiwa 1st 5/1.
    
    Any date in the year during which era is changed can be reckoned in either era. For example,
    2019/1/1 can be 1/1 Reiwa 1st year or 1/1 Heisei 31st year.
    
    Note:
     The DateTime can be represented by the JapaneseCalendar are limited to two factors:
         1. The min value and max value of DateTime class.
         2. The available era information.
    """

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def __init__(self) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        ...

    def get_week_of_year(self, time: typing.Union[datetime.datetime, datetime.date], rule: System.Globalization.CalendarWeekRule, first_day_of_week: System.DayOfWeek) -> int:
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        ...

    def to_four_digit_year(self, year: int) -> int:
        """
        For Japanese calendar, four digit year is not used. Few emperors will live for more than one hundred years.
        Therefore, for any two digit number, we just return the original number.
        """
        ...


class TextInfo(System.Object, System.ICloneable, System.Runtime.Serialization.IDeserializationCallback):
    """
    This Class defines behaviors specific to a writing system.
    A writing system is the collection of scripts and orthographic rules
    required to represent a language as text.
    """

    @property
    def ansi_code_page(self) -> int:
        ...

    @property
    def oem_code_page(self) -> int:
        ...

    @property
    def mac_code_page(self) -> int:
        ...

    @property
    def ebcdic_code_page(self) -> int:
        ...

    @property
    def lcid(self) -> int:
        ...

    @property
    def culture_name(self) -> str:
        ...

    @property
    def is_read_only(self) -> bool:
        ...

    @property
    def list_separator(self) -> str:
        """Returns the string used to separate items in a list."""
        ...

    @property.setter
    def list_separator(self, value: str) -> None:
        ...

    @property
    def is_right_to_left(self) -> bool:
        """
        Returns true if the dominant direction of text and UI such as the
        relative position of buttons and scroll bars
        """
        ...

    def clone(self) -> System.Object:
        ...

    def equals(self, obj: typing.Any) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    @staticmethod
    def read_only(text_info: System.Globalization.TextInfo) -> System.Globalization.TextInfo:
        """
        Create a cloned readonly instance or return the input one if it is
        readonly.
        """
        ...

    @overload
    def to_lower(self, c: str) -> str:
        """
        Converts the character or string to lower case.  Certain locales
        have different casing semantics from the file systems in Win32.
        """
        ...

    @overload
    def to_lower(self, str: str) -> str:
        ...

    def to_string(self) -> str:
        ...

    def to_title_case(self, str: str) -> str:
        """
        Titlecasing refers to a casing practice wherein the first letter of a word is an uppercase letter
        and the rest of the letters are lowercase.  The choice of which words to titlecase in headings
        and titles is dependent on language and local conventions.  For example, "The Merry Wives of Windor"
        is the appropriate titlecasing of that play's name in English, with the word "of" not titlecased.
        In German, however, the title is "Die lustigen Weiber von Windsor," and both "lustigen" and "von"
        are not titlecased.  In French even fewer words are titlecased: "Les joyeuses commeres de Windsor."
        
        Moreover, the determination of what actually constitutes a word is language dependent, and this can
        influence which letter or letters of a "word" are uppercased when titlecasing strings.  For example
        "l'arbre" is considered two words in French, whereas "can't" is considered one word in English.
        """
        ...

    @overload
    def to_upper(self, c: str) -> str:
        """
        Converts the character or string to upper case.  Certain locales
        have different casing semantics from the file systems in Win32.
        """
        ...

    @overload
    def to_upper(self, str: str) -> str:
        ...


class TextElementEnumerator(System.Object, System.Collections.IEnumerator):
    """This class has no documentation."""

    @property
    def current(self) -> System.Object:
        ...

    @property
    def element_index(self) -> int:
        ...

    def get_text_element(self) -> str:
        ...

    def move_next(self) -> bool:
        ...

    def reset(self) -> None:
        ...


class CultureTypes(Enum):
    """This class has no documentation."""

    NEUTRAL_CULTURES = ...

    SPECIFIC_CULTURES = ...

    INSTALLED_WIN_32_CULTURES = ...

    ALL_CULTURES = ...

    USER_CUSTOM_CULTURE = ...

    REPLACEMENT_CULTURES = ...

    WINDOWS_ONLY_CULTURES = ...
    """CultureTypes.WindowsOnlyCultures has been deprecated. Use other values in CultureTypes instead."""

    FRAMEWORK_CULTURES = ...
    """CultureTypes.FrameworkCultures has been deprecated. Use other values in CultureTypes instead."""


class UnicodeCategory(Enum):
    """This class has no documentation."""

    UPPERCASE_LETTER = 0

    LOWERCASE_LETTER = 1

    TITLECASE_LETTER = 2

    MODIFIER_LETTER = 3

    OTHER_LETTER = 4

    NON_SPACING_MARK = 5

    SPACING_COMBINING_MARK = 6

    ENCLOSING_MARK = 7

    DECIMAL_DIGIT_NUMBER = 8

    LETTER_NUMBER = 9

    OTHER_NUMBER = 10

    SPACE_SEPARATOR = 11

    LINE_SEPARATOR = 12

    PARAGRAPH_SEPARATOR = 13

    CONTROL = 14

    FORMAT = 15

    SURROGATE = 16

    PRIVATE_USE = 17

    CONNECTOR_PUNCTUATION = 18

    DASH_PUNCTUATION = 19

    OPEN_PUNCTUATION = 20

    CLOSE_PUNCTUATION = 21

    INITIAL_QUOTE_PUNCTUATION = 22

    FINAL_QUOTE_PUNCTUATION = 23

    OTHER_PUNCTUATION = 24

    MATH_SYMBOL = 25

    CURRENCY_SYMBOL = 26

    MODIFIER_SYMBOL = 27

    OTHER_SYMBOL = 28

    OTHER_NOT_ASSIGNED = 29


class CharUnicodeInfo(System.Object):
    """
    This class implements a set of methods for retrieving character type
    information. Character type information is independent of culture
    and region.
    """

    @staticmethod
    @overload
    def get_decimal_digit_value(ch: str) -> int:
        ...

    @staticmethod
    @overload
    def get_decimal_digit_value(s: str, index: int) -> int:
        ...

    @staticmethod
    @overload
    def get_digit_value(ch: str) -> int:
        ...

    @staticmethod
    @overload
    def get_digit_value(s: str, index: int) -> int:
        ...

    @staticmethod
    @overload
    def get_numeric_value(ch: str) -> float:
        ...

    @staticmethod
    @overload
    def get_numeric_value(s: str, index: int) -> float:
        ...

    @staticmethod
    @overload
    def get_unicode_category(ch: str) -> System.Globalization.UnicodeCategory:
        ...

    @staticmethod
    @overload
    def get_unicode_category(code_point: int) -> System.Globalization.UnicodeCategory:
        ...

    @staticmethod
    @overload
    def get_unicode_category(s: str, index: int) -> System.Globalization.UnicodeCategory:
        ...


class JapaneseLunisolarCalendar(System.Globalization.EastAsianLunisolarCalendar):
    """This class has no documentation."""

    JAPANESE_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def days_in_year_before_min_supported_year(self) -> int:
        """This property is protected."""
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    def __init__(self) -> None:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...


class DigitShapes(Enum):
    """This class has no documentation."""

    CONTEXT = ...

    NONE = ...

    NATIVE_NATIONAL = ...


class NumberFormatInfo(System.Object, System.IFormatProvider, System.ICloneable):
    """This class has no documentation."""

    INVARIANT_INFO: System.Globalization.NumberFormatInfo
    """
    Returns a default NumberFormatInfo that will be universally
    supported and constant irrespective of the current culture.
    Used by FromString methods.
    """

    @property
    def currency_decimal_digits(self) -> int:
        ...

    @property.setter
    def currency_decimal_digits(self, value: int) -> None:
        ...

    @property
    def currency_decimal_separator(self) -> str:
        ...

    @property.setter
    def currency_decimal_separator(self, value: str) -> None:
        ...

    @property
    def is_read_only(self) -> bool:
        ...

    @property
    def currency_group_sizes(self) -> typing.List[int]:
        ...

    @property.setter
    def currency_group_sizes(self, value: typing.List[int]) -> None:
        ...

    @property
    def number_group_sizes(self) -> typing.List[int]:
        ...

    @property.setter
    def number_group_sizes(self, value: typing.List[int]) -> None:
        ...

    @property
    def percent_group_sizes(self) -> typing.List[int]:
        ...

    @property.setter
    def percent_group_sizes(self, value: typing.List[int]) -> None:
        ...

    @property
    def currency_group_separator(self) -> str:
        ...

    @property.setter
    def currency_group_separator(self, value: str) -> None:
        ...

    @property
    def currency_symbol(self) -> str:
        ...

    @property.setter
    def currency_symbol(self, value: str) -> None:
        ...

    CURRENT_INFO: System.Globalization.NumberFormatInfo
    """Returns the current culture's NumberFormatInfo. Used by Parse methods."""

    @property
    def na_n_symbol(self) -> str:
        ...

    @property.setter
    def na_n_symbol(self, value: str) -> None:
        ...

    @property
    def currency_negative_pattern(self) -> int:
        ...

    @property.setter
    def currency_negative_pattern(self, value: int) -> None:
        ...

    @property
    def number_negative_pattern(self) -> int:
        ...

    @property.setter
    def number_negative_pattern(self, value: int) -> None:
        ...

    @property
    def percent_positive_pattern(self) -> int:
        ...

    @property.setter
    def percent_positive_pattern(self, value: int) -> None:
        ...

    @property
    def percent_negative_pattern(self) -> int:
        ...

    @property.setter
    def percent_negative_pattern(self, value: int) -> None:
        ...

    @property
    def negative_infinity_symbol(self) -> str:
        ...

    @property.setter
    def negative_infinity_symbol(self, value: str) -> None:
        ...

    @property
    def negative_sign(self) -> str:
        ...

    @property.setter
    def negative_sign(self, value: str) -> None:
        ...

    @property
    def number_decimal_digits(self) -> int:
        ...

    @property.setter
    def number_decimal_digits(self, value: int) -> None:
        ...

    @property
    def number_decimal_separator(self) -> str:
        ...

    @property.setter
    def number_decimal_separator(self, value: str) -> None:
        ...

    @property
    def number_group_separator(self) -> str:
        ...

    @property.setter
    def number_group_separator(self, value: str) -> None:
        ...

    @property
    def currency_positive_pattern(self) -> int:
        ...

    @property.setter
    def currency_positive_pattern(self, value: int) -> None:
        ...

    @property
    def positive_infinity_symbol(self) -> str:
        ...

    @property.setter
    def positive_infinity_symbol(self, value: str) -> None:
        ...

    @property
    def positive_sign(self) -> str:
        ...

    @property.setter
    def positive_sign(self, value: str) -> None:
        ...

    @property
    def percent_decimal_digits(self) -> int:
        ...

    @property.setter
    def percent_decimal_digits(self, value: int) -> None:
        ...

    @property
    def percent_decimal_separator(self) -> str:
        ...

    @property.setter
    def percent_decimal_separator(self, value: str) -> None:
        ...

    @property
    def percent_group_separator(self) -> str:
        ...

    @property.setter
    def percent_group_separator(self, value: str) -> None:
        ...

    @property
    def percent_symbol(self) -> str:
        ...

    @property.setter
    def percent_symbol(self, value: str) -> None:
        ...

    @property
    def per_mille_symbol(self) -> str:
        ...

    @property.setter
    def per_mille_symbol(self, value: str) -> None:
        ...

    @property
    def native_digits(self) -> typing.List[str]:
        ...

    @property.setter
    def native_digits(self, value: typing.List[str]) -> None:
        ...

    @property
    def digit_substitution(self) -> System.Globalization.DigitShapes:
        ...

    @property.setter
    def digit_substitution(self, value: System.Globalization.DigitShapes) -> None:
        ...

    def __init__(self) -> None:
        ...

    def clone(self) -> System.Object:
        ...

    def get_format(self, format_type: typing.Type) -> System.Object:
        ...

    @staticmethod
    def get_instance(format_provider: System.IFormatProvider) -> System.Globalization.NumberFormatInfo:
        ...

    @staticmethod
    def read_only(nfi: System.Globalization.NumberFormatInfo) -> System.Globalization.NumberFormatInfo:
        ...


class GlobalizationExtensions(System.Object):
    """This class has no documentation."""

    @staticmethod
    def get_string_comparer(compare_info: System.Globalization.CompareInfo, options: System.Globalization.CompareOptions) -> System.StringComparer:
        ...


class HijriCalendar(System.Globalization.Calendar):
    """This class has no documentation."""

    HIJRI_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def days_in_year_before_min_supported_year(self) -> int:
        """This property is protected."""
        ...

    @property
    def hijri_adjustment(self) -> int:
        ...

    @property.setter
    def hijri_adjustment(self, value: int) -> None:
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def __init__(self) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        ...

    def to_four_digit_year(self, year: int) -> int:
        ...


class TimeSpanStyles(Enum):
    """This class has no documentation."""

    NONE = ...

    ASSUME_NEGATIVE = ...


class DateTimeFormatInfo(System.Object, System.IFormatProvider, System.ICloneable):
    """This class has no documentation."""

    INVARIANT_INFO: System.Globalization.DateTimeFormatInfo
    """
    Returns a default DateTimeFormatInfo that will be universally
    supported and constant irrespective of the current culture.
    """

    CURRENT_INFO: System.Globalization.DateTimeFormatInfo
    """Returns the current culture's DateTimeFormatInfo."""

    @property
    def am_designator(self) -> str:
        ...

    @property.setter
    def am_designator(self, value: str) -> None:
        ...

    @property
    def calendar(self) -> System.Globalization.Calendar:
        ...

    @property.setter
    def calendar(self, value: System.Globalization.Calendar) -> None:
        ...

    @property
    def date_separator(self) -> str:
        ...

    @property.setter
    def date_separator(self, value: str) -> None:
        ...

    @property
    def first_day_of_week(self) -> System.DayOfWeek:
        ...

    @property.setter
    def first_day_of_week(self, value: System.DayOfWeek) -> None:
        ...

    @property
    def calendar_week_rule(self) -> System.Globalization.CalendarWeekRule:
        ...

    @property.setter
    def calendar_week_rule(self, value: System.Globalization.CalendarWeekRule) -> None:
        ...

    @property
    def full_date_time_pattern(self) -> str:
        ...

    @property.setter
    def full_date_time_pattern(self, value: str) -> None:
        ...

    @property
    def long_date_pattern(self) -> str:
        """
        For our "patterns" arrays we have 2 variables, a string and a string[]
        The string[] contains the list of patterns, EXCEPT the default may not be included.
        The string contains the default pattern.
        When we initially construct our string[], we set the string to string[0]
        """
        ...

    @property.setter
    def long_date_pattern(self, value: str) -> None:
        ...

    @property
    def long_time_pattern(self) -> str:
        """
        For our "patterns" arrays we have 2 variables, a string and a string[]
        
        The string[] contains the list of patterns, EXCEPT the default may not be included.
        The string contains the default pattern.
        When we initially construct our string[], we set the string to string[0]
        """
        ...

    @property.setter
    def long_time_pattern(self, value: str) -> None:
        ...

    @property
    def month_day_pattern(self) -> str:
        ...

    @property.setter
    def month_day_pattern(self, value: str) -> None:
        ...

    @property
    def pm_designator(self) -> str:
        ...

    @property.setter
    def pm_designator(self, value: str) -> None:
        ...

    @property
    def rfc_1123_pattern(self) -> str:
        ...

    @property
    def short_date_pattern(self) -> str:
        """
        For our "patterns" arrays we have 2 variables, a string and a string[]
        
        The string[] contains the list of patterns, EXCEPT the default may not be included.
        The string contains the default pattern.
        When we initially construct our string[], we set the string to string[0]
        """
        ...

    @property.setter
    def short_date_pattern(self, value: str) -> None:
        ...

    @property
    def short_time_pattern(self) -> str:
        """
        For our "patterns" arrays we have 2 variables, a string and a string[]
        
        The string[] contains the list of patterns, EXCEPT the default may not be included.
        The string contains the default pattern.
        When we initially construct our string[], we set the string to string[0]
        """
        ...

    @property.setter
    def short_time_pattern(self, value: str) -> None:
        ...

    @property
    def sortable_date_time_pattern(self) -> str:
        ...

    @property
    def time_separator(self) -> str:
        ...

    @property.setter
    def time_separator(self, value: str) -> None:
        ...

    @property
    def universal_sortable_date_time_pattern(self) -> str:
        ...

    @property
    def year_month_pattern(self) -> str:
        """
        For our "patterns" arrays we have 2 variables, a string and a string[]
        
        The string[] contains the list of patterns, EXCEPT the default may not be included.
        The string contains the default pattern.
        When we initially construct our string[], we set the string to string[0]
        """
        ...

    @property.setter
    def year_month_pattern(self, value: str) -> None:
        ...

    @property
    def abbreviated_day_names(self) -> typing.List[str]:
        ...

    @property.setter
    def abbreviated_day_names(self, value: typing.List[str]) -> None:
        ...

    @property
    def shortest_day_names(self) -> typing.List[str]:
        """Returns the string array of the one-letter day of week names."""
        ...

    @property.setter
    def shortest_day_names(self, value: typing.List[str]) -> None:
        ...

    @property
    def day_names(self) -> typing.List[str]:
        ...

    @property.setter
    def day_names(self, value: typing.List[str]) -> None:
        ...

    @property
    def abbreviated_month_names(self) -> typing.List[str]:
        ...

    @property.setter
    def abbreviated_month_names(self, value: typing.List[str]) -> None:
        ...

    @property
    def month_names(self) -> typing.List[str]:
        ...

    @property.setter
    def month_names(self, value: typing.List[str]) -> None:
        ...

    @property
    def is_read_only(self) -> bool:
        ...

    @property
    def native_calendar_name(self) -> str:
        """
        Return the native name for the calendar in DTFI.Calendar.  The native name is referred to
        the culture used to create the DTFI.  E.g. in the following example, the native language is Japanese.
        DateTimeFormatInfo dtfi = new CultureInfo("ja-JP", false).DateTimeFormat.Calendar = new JapaneseCalendar();
        String nativeName = dtfi.NativeCalendarName; // Get the Japanese name for the Japanese calendar.
        DateTimeFormatInfo dtfi = new CultureInfo("ja-JP", false).DateTimeFormat.Calendar = new GregorianCalendar(GregorianCalendarTypes.Localized);
        String nativeName = dtfi.NativeCalendarName; // Get the Japanese name for the Gregorian calendar.
        """
        ...

    @property
    def abbreviated_month_genitive_names(self) -> typing.List[str]:
        ...

    @property.setter
    def abbreviated_month_genitive_names(self, value: typing.List[str]) -> None:
        ...

    @property
    def month_genitive_names(self) -> typing.List[str]:
        ...

    @property.setter
    def month_genitive_names(self, value: typing.List[str]) -> None:
        ...

    def __init__(self) -> None:
        ...

    def clone(self) -> System.Object:
        ...

    def get_abbreviated_day_name(self, dayofweek: System.DayOfWeek) -> str:
        ...

    def get_abbreviated_era_name(self, era: int) -> str:
        ...

    def get_abbreviated_month_name(self, month: int) -> str:
        ...

    @overload
    def get_all_date_time_patterns(self) -> typing.List[str]:
        ...

    @overload
    def get_all_date_time_patterns(self, format: str) -> typing.List[str]:
        ...

    def get_day_name(self, dayofweek: System.DayOfWeek) -> str:
        ...

    def get_era(self, era_name: str) -> int:
        """Get the era value by parsing the name of the era."""
        ...

    def get_era_name(self, era: int) -> str:
        """
        Get the name of the era for the specified era value.
        Era names are 1 indexed
        """
        ...

    def get_format(self, format_type: typing.Type) -> System.Object:
        ...

    @staticmethod
    def get_instance(provider: System.IFormatProvider) -> System.Globalization.DateTimeFormatInfo:
        ...

    def get_month_name(self, month: int) -> str:
        ...

    def get_shortest_day_name(self, day_of_week: System.DayOfWeek) -> str:
        """Returns the super short day of week names for the specified day of week."""
        ...

    @staticmethod
    def read_only(dtfi: System.Globalization.DateTimeFormatInfo) -> System.Globalization.DateTimeFormatInfo:
        ...

    def set_all_date_time_patterns(self, patterns: typing.List[str], format: str) -> None:
        """
        Used by custom cultures and others to set the list of available formats. Note that none of them are
        explicitly used unless someone calls GetAllDateTimePatterns and subsequently uses one of the items
        from the list.
        
        Most of the format characters that can be used in GetAllDateTimePatterns are
        not really needed since they are one of the following:
        
         r/R/s/u     locale-independent constants -- cannot be changed!
         m/M/y/Y     fields with a single string in them -- that can be set through props directly
         f/F/g/G/U   derived fields based on combinations of various of the below formats
        
        NOTE: No special validation is done here beyond what is done when the actual respective fields
        are used (what would be the point of disallowing here what we allow in the appropriate property?)
        
        WARNING: If more validation is ever done in one place, it should be done in the other.
        """
        ...


class CultureInfo(System.Object, System.IFormatProvider, System.ICloneable):
    """
    This class represents the software preferences of a particular culture
    or community. It includes information such as the language, writing
    system and a calendar used by the culture as well as methods for
    common operations such as printing dates and sorting strings.
    """

    current_culture: System.Globalization.CultureInfo
    """
    This instance provides methods based on the current user settings.
    These settings are volatile and may change over the lifetime of the
    thread.
    """

    current_ui_culture: System.Globalization.CultureInfo

    INSTALLED_UI_CULTURE: System.Globalization.CultureInfo

    default_thread_current_culture: System.Globalization.CultureInfo

    default_thread_current_ui_culture: System.Globalization.CultureInfo

    INVARIANT_CULTURE: System.Globalization.CultureInfo
    """
    This instance provides methods, for example for casing and sorting,
    that are independent of the system and current user settings.  It
    should be used only by processes such as some system services that
    require such invariant results (eg. file systems).  In general,
    the results are not linguistically correct and do not match any
    culture info.
    """

    @property
    def parent(self) -> System.Globalization.CultureInfo:
        """Return the parent CultureInfo for the current instance."""
        ...

    @property
    def lcid(self) -> int:
        ...

    @property
    def keyboard_layout_id(self) -> int:
        ...

    @property
    def name(self) -> str:
        """
        Returns the full name of the CultureInfo. The name is in format like
        "en-US" This version does NOT include sort information in the name.
        """
        ...

    @property
    def ietf_language_tag(self) -> str:
        ...

    @property
    def display_name(self) -> str:
        """
        Returns the full name of the CultureInfo in the localized language.
        For example, if the localized language of the runtime is Spanish and the CultureInfo is
        US English, "Ingles (Estados Unidos)" will be returned.
        """
        ...

    @property
    def native_name(self) -> str:
        """
        Returns the full name of the CultureInfo in the native language.
        For example, if the CultureInfo is US English, "English
        (United States)" will be returned.
        """
        ...

    @property
    def english_name(self) -> str:
        """
        Returns the full name of the CultureInfo in English.
        For example, if the CultureInfo is US English, "English
        (United States)" will be returned.
        """
        ...

    @property
    def two_letter_iso_language_name(self) -> str:
        """ie: en"""
        ...

    @property
    def three_letter_iso_language_name(self) -> str:
        """ie: eng"""
        ...

    @property
    def three_letter_windows_language_name(self) -> str:
        """
        Returns the 3 letter windows language name for the current instance.  eg: "ENU"
        The ISO names are much preferred
        """
        ...

    @property
    def compare_info(self) -> System.Globalization.CompareInfo:
        """Gets the CompareInfo for this culture."""
        ...

    @property
    def text_info(self) -> System.Globalization.TextInfo:
        """Gets the TextInfo for this culture."""
        ...

    @property
    def is_neutral_culture(self) -> bool:
        ...

    @property
    def culture_types(self) -> System.Globalization.CultureTypes:
        ...

    @property
    def number_format(self) -> System.Globalization.NumberFormatInfo:
        ...

    @property.setter
    def number_format(self, value: System.Globalization.NumberFormatInfo) -> None:
        ...

    @property
    def date_time_format(self) -> System.Globalization.DateTimeFormatInfo:
        """
        Create a DateTimeFormatInfo, and fill in the properties according to
        the CultureID.
        """
        ...

    @property.setter
    def date_time_format(self, value: System.Globalization.DateTimeFormatInfo) -> None:
        ...

    @property
    def calendar(self) -> System.Globalization.Calendar:
        """
        Return/set the default calendar used by this culture.
        This value can be overridden by regional option if this is a current culture.
        """
        ...

    @property
    def optional_calendars(self) -> typing.List[System.Globalization.Calendar]:
        """Return an array of the optional calendar for this culture."""
        ...

    @property
    def use_user_override(self) -> bool:
        ...

    @property
    def is_read_only(self) -> bool:
        ...

    @overload
    def __init__(self, name: str) -> None:
        ...

    @overload
    def __init__(self, name: str, useUserOverride: bool) -> None:
        ...

    @overload
    def __init__(self, culture: int) -> None:
        ...

    @overload
    def __init__(self, culture: int, useUserOverride: bool) -> None:
        ...

    def clear_cached_data(self) -> None:
        ...

    def clone(self) -> System.Object:
        ...

    @staticmethod
    def create_specific_culture(name: str) -> System.Globalization.CultureInfo:
        """
        Return a specific culture. A tad irrelevant now since we always
        return valid data for neutral locales.
        
        Note that there's interesting behavior that tries to find a
        smaller name, ala RFC4647, if we can't find a bigger name.
        That doesn't help with things like "zh" though, so the approach
        is of questionable value
        """
        ...

    def equals(self, value: typing.Any) -> bool:
        ...

    def get_console_fallback_ui_culture(self) -> System.Globalization.CultureInfo:
        ...

    @staticmethod
    @overload
    def get_culture_info(culture: int) -> System.Globalization.CultureInfo:
        """
        Gets a cached copy of the specified culture from an internal
        hashtable (or creates it if not found). (LCID version)
        """
        ...

    @staticmethod
    @overload
    def get_culture_info(name: str) -> System.Globalization.CultureInfo:
        """
        Gets a cached copy of the specified culture from an internal
        hashtable (or creates it if not found). (Named version)
        """
        ...

    @staticmethod
    @overload
    def get_culture_info(name: str, alt_name: str) -> System.Globalization.CultureInfo:
        """
        Gets a cached copy of the specified culture from an internal
        hashtable (or creates it if not found).
        """
        ...

    @staticmethod
    @overload
    def get_culture_info(name: str, predefined_only: bool) -> System.Globalization.CultureInfo:
        ...

    @staticmethod
    def get_culture_info_by_ietf_language_tag(name: str) -> System.Globalization.CultureInfo:
        ...

    @staticmethod
    def get_cultures(types: System.Globalization.CultureTypes) -> typing.List[System.Globalization.CultureInfo]:
        ...

    def get_format(self, format_type: typing.Type) -> System.Object:
        ...

    def get_hash_code(self) -> int:
        ...

    @staticmethod
    def read_only(ci: System.Globalization.CultureInfo) -> System.Globalization.CultureInfo:
        ...

    def to_string(self) -> str:
        """
        Implements object.ToString(). Returns the name of the CultureInfo,
        eg. "de-DE_phoneb", "en-US", or "fj-FJ".
        """
        ...


class ChineseLunisolarCalendar(System.Globalization.EastAsianLunisolarCalendar):
    """This class has no documentation."""

    CHINESE_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def days_in_year_before_min_supported_year(self) -> int:
        """This property is protected."""
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    def __init__(self) -> None:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...


class ThaiBuddhistCalendar(System.Globalization.Calendar):
    """
    ThaiBuddhistCalendar is based on Gregorian calendar.
    Its year value has an offset to the Gregorain calendar.
    """

    THAI_BUDDHIST_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def __init__(self) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        ...

    def get_week_of_year(self, time: typing.Union[datetime.datetime, datetime.date], rule: System.Globalization.CalendarWeekRule, first_day_of_week: System.DayOfWeek) -> int:
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        ...

    def to_four_digit_year(self, year: int) -> int:
        ...


class JulianCalendar(System.Globalization.Calendar):
    """
    This class implements the Julian calendar. In 48 B.C. Julius Caesar
    ordered a calendar reform, and this calendar is called Julian calendar.
    It consisted of a solar year of twelve months and of 365 days with an
    extra day every fourth year.
    """

    JULIAN_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def __init__(self) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        ...

    def to_four_digit_year(self, year: int) -> int:
        ...


class StringInfo(System.Object):
    """
    This class defines behaviors specific to a writing system.
    A writing system is the collection of scripts and orthographic rules
    required to represent a language as text.
    """

    @property
    def string(self) -> str:
        ...

    @property.setter
    def string(self, value: str) -> None:
        ...

    @property
    def length_in_text_elements(self) -> int:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, value: str) -> None:
        ...

    def equals(self, value: typing.Any) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    @staticmethod
    @overload
    def get_next_text_element(str: str) -> str:
        """
        Returns the first text element (extended grapheme cluster) that occurs in the input string.
        
        :param str: The input string to analyze.
        :returns: The substring corresponding to the first text element within , or the empty string if  is empty.
        """
        ...

    @staticmethod
    @overload
    def get_next_text_element(str: str, index: int) -> str:
        """
        Returns the first text element (extended grapheme cluster) that occurs in the input string
        starting at the specified index.
        
        :param str: The input string to analyze.
        :param index: The char offset in  at which to begin analysis.
        :returns: The substring corresponding to the first text element within  starting at index , or the empty string if  corresponds to the end of .
        """
        ...

    @staticmethod
    @overload
    def get_next_text_element_length(str: str) -> int:
        """
        Returns the length of the first text element (extended grapheme cluster) that occurs in the input string.
        
        :param str: The input string to analyze.
        :returns: The length (in chars) of the substring corresponding to the first text element within , or 0 if  is empty.
        """
        ...

    @staticmethod
    @overload
    def get_next_text_element_length(str: str, index: int) -> int:
        """
        Returns the length of the first text element (extended grapheme cluster) that occurs in the input string
        starting at the specified index.
        
        :param str: The input string to analyze.
        :param index: The char offset in  at which to begin analysis.
        :returns: The length (in chars) of the substring corresponding to the first text element within  starting at index , or 0 if  corresponds to the end of .
        """
        ...

    @staticmethod
    @overload
    def get_next_text_element_length(str: System.ReadOnlySpan[str]) -> int:
        """
        Returns the length of the first text element (extended grapheme cluster) that occurs in the input span.
        
        :param str: The input span to analyze.
        :returns: The length (in chars) of the substring corresponding to the first text element within , or 0 if  is empty.
        """
        ...

    @staticmethod
    @overload
    def get_text_element_enumerator(str: str) -> System.Globalization.TextElementEnumerator:
        ...

    @staticmethod
    @overload
    def get_text_element_enumerator(str: str, index: int) -> System.Globalization.TextElementEnumerator:
        ...

    @staticmethod
    def parse_combining_characters(str: str) -> typing.List[int]:
        """
        Returns the indices of each base character or properly formed surrogate
        pair  within the str. It recognizes a base character plus one or more
        combining characters or a properly formed surrogate pair as a text
        element and returns the index of the base character or high surrogate.
        Each index is the beginning of a text element within a str. The length
        of each element is easily computed as the difference between successive
        indices. The length of the array will always be less than or equal to
        the length of the str. For example, given the str
        \\u4f00\\u302a\\ud800\\udc00\\u4f01, this method would return the indices:
        0, 2, 4.
        """
        ...

    @overload
    def substring_by_text_elements(self, starting_text_element: int) -> str:
        ...

    @overload
    def substring_by_text_elements(self, starting_text_element: int, length_in_text_elements: int) -> str:
        ...


class KoreanLunisolarCalendar(System.Globalization.EastAsianLunisolarCalendar):
    """This class has no documentation."""

    GREGORIAN_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def days_in_year_before_min_supported_year(self) -> int:
        """This property is protected."""
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    def __init__(self) -> None:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...


class ISOWeek(System.Object):
    """This class has no documentation."""

    @staticmethod
    @overload
    def get_week_of_year(date: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    @staticmethod
    @overload
    def get_week_of_year(date: System.DateOnly) -> int:
        """
        Calculates the ISO week number of a given Gregorian date.
        
        :param date: A date in the Gregorian calendar.
        :returns: A number between 1 and 53 representing the ISO week number of the given Gregorian date.
        """
        ...

    @staticmethod
    def get_weeks_in_year(year: int) -> int:
        ...

    @staticmethod
    @overload
    def get_year(date: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    @staticmethod
    @overload
    def get_year(date: System.DateOnly) -> int:
        """
        Calculates the ISO week-numbering year (also called ISO year informally) mapped to the input Gregorian date.
        
        :param date: A date in the Gregorian calendar.
        :returns: The ISO week-numbering year, between 1 and 9999.
        """
        ...

    @staticmethod
    def get_year_end(year: int) -> datetime.datetime:
        ...

    @staticmethod
    def get_year_start(year: int) -> datetime.datetime:
        ...

    @staticmethod
    def to_date_only(year: int, week: int, day_of_week: System.DayOfWeek) -> System.DateOnly:
        """
        Maps the ISO week date represented by a specified ISO year, week number, and day of week to the equivalent Gregorian date.
        
        :param year: An ISO week-numbering year (also called an ISO year informally).
        :param week: The ISO week number in the given ISO week-numbering year.
        :param day_of_week: The day of week inside the given ISO week.
        :returns: The Gregorian date equivalent to the input ISO week date.
        """
        ...

    @staticmethod
    def to_date_time(year: int, week: int, day_of_week: System.DayOfWeek) -> datetime.datetime:
        ...


class CultureNotFoundException(System.ArgumentException):
    """This class has no documentation."""

    @property
    def invalid_culture_id(self) -> typing.Optional[int]:
        ...

    @property
    def invalid_culture_name(self) -> str:
        ...

    @property
    def message(self) -> str:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, message: str) -> None:
        ...

    @overload
    def __init__(self, paramName: str, message: str) -> None:
        ...

    @overload
    def __init__(self, message: str, innerException: System.Exception) -> None:
        ...

    @overload
    def __init__(self, paramName: str, invalidCultureName: str, message: str) -> None:
        ...

    @overload
    def __init__(self, message: str, invalidCultureName: str, innerException: System.Exception) -> None:
        ...

    @overload
    def __init__(self, message: str, invalidCultureId: int, innerException: System.Exception) -> None:
        ...

    @overload
    def __init__(self, paramName: str, invalidCultureId: int, message: str) -> None:
        ...

    @overload
    def __init__(self, info: System.Runtime.Serialization.SerializationInfo, context: System.Runtime.Serialization.StreamingContext) -> None:
        """
        This method is protected.
        
        Obsoletions.LegacyFormatterImplMessage
        """
        ...

    def get_object_data(self, info: System.Runtime.Serialization.SerializationInfo, context: System.Runtime.Serialization.StreamingContext) -> None:
        """Obsoletions.LegacyFormatterImplMessage"""
        warnings.warn("Obsoletions.LegacyFormatterImplMessage", DeprecationWarning)


class PersianCalendar(System.Globalization.Calendar):
    """
    Modern Persian calendar is a solar observation based calendar. Each new year begins on the day when the vernal equinox occurs before noon.
    The epoch is the date of the vernal equinox prior to the epoch of the Islamic calendar (March 19, 622 Julian or March 22, 622 Gregorian)
    There is no Persian year 0. Ordinary years have 365 days. Leap years have 366 days with the last month (Esfand) gaining the extra day.
    """

    PERSIAN_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def __init__(self) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        ...

    def to_four_digit_year(self, year: int) -> int:
        ...


class UmAlQuraCalendar(System.Globalization.Calendar):
    """This class has no documentation."""

    UM_AL_QURA_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def days_in_year_before_min_supported_year(self) -> int:
        """This property is protected."""
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def __init__(self) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        ...

    def to_four_digit_year(self, year: int) -> int:
        ...


class HebrewCalendar(System.Globalization.Calendar):
    """This class has no documentation."""

    HEBREW_ERA: int = 1

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def __init__(self) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        ...

    def to_four_digit_year(self, year: int) -> int:
        ...


class RegionInfo(System.Object):
    """
    This class represents settings specified by de jure or de facto
    standards for a particular country/region. In contrast to
    CultureInfo, the RegionInfo does not represent preferences of the
    user and does not depend on the user's language or culture.
    """

    CURRENT_REGION: System.Globalization.RegionInfo
    """This instance provides methods based on the current user settings."""

    @property
    def name(self) -> str:
        """Returns the name of the region (ie: en-US)"""
        ...

    @property
    def english_name(self) -> str:
        """Returns the name of the region in English. (ie: United States)"""
        ...

    @property
    def display_name(self) -> str:
        """
        Returns the display name (localized) of the region. (ie: United States
        if the current UI language is en-US)
        """
        ...

    @property
    def native_name(self) -> str:
        """
        Returns the native name of the region. (ie: Deutschland)
         WARNING: You need a full locale name for this to make sense.
        """
        ...

    @property
    def two_letter_iso_region_name(self) -> str:
        """Returns the two letter ISO region name (ie: US)"""
        ...

    @property
    def three_letter_iso_region_name(self) -> str:
        """Returns the three letter ISO region name (ie: USA)"""
        ...

    @property
    def three_letter_windows_region_name(self) -> str:
        """Returns the three letter windows region name (ie: USA)"""
        ...

    @property
    def is_metric(self) -> bool:
        """Returns true if this region uses the metric measurement system"""
        ...

    @property
    def geo_id(self) -> int:
        ...

    @property
    def currency_english_name(self) -> str:
        """English name for this region's currency, ie: Swiss Franc"""
        ...

    @property
    def currency_native_name(self) -> str:
        """
        Native name for this region's currency, ie: Schweizer Franken
        WARNING: You need a full locale name for this to make sense.
        """
        ...

    @property
    def currency_symbol(self) -> str:
        """Currency Symbol for this locale, ie: Fr. or $"""
        ...

    @property
    def iso_currency_symbol(self) -> str:
        """ISO Currency Symbol for this locale, ie: CHF"""
        ...

    @overload
    def __init__(self, name: str) -> None:
        ...

    @overload
    def __init__(self, culture: int) -> None:
        ...

    def equals(self, value: typing.Any) -> bool:
        """
        Implements Object.Equals().  Returns a boolean indicating whether
        or not object refers to the same RegionInfo as the current instance.
        RegionInfos are considered equal if and only if they have the same name
        (ie: en-US)
        """
        ...

    def get_hash_code(self) -> int:
        ...

    def to_string(self) -> str:
        ...


class NumberStyles(Enum):
    """
    Contains valid formats for Numbers recognized by the Number
    class' parsing code.
    """

    NONE = ...

    ALLOW_LEADING_WHITE = ...
    """
    Bit flag indicating that leading whitespace is allowed. Character values
    0x0009, 0x000A, 0x000B, 0x000C, 0x000D, and 0x0020 are considered to be
    whitespace.
    """

    ALLOW_TRAILING_WHITE = ...
    """Bitflag indicating trailing whitespace is allowed."""

    ALLOW_LEADING_SIGN = ...
    """
    Can the number start with a sign char specified by
    NumberFormatInfo.PositiveSign and NumberFormatInfo.NegativeSign
    """

    ALLOW_TRAILING_SIGN = ...
    """Allow the number to end with a sign char"""

    ALLOW_PARENTHESES = ...
    """Allow the number to be enclosed in parens"""

    ALLOW_DECIMAL_POINT = ...

    ALLOW_THOUSANDS = ...

    ALLOW_EXPONENT = ...

    ALLOW_CURRENCY_SYMBOL = ...

    ALLOW_HEX_SPECIFIER = ...

    ALLOW_BINARY_SPECIFIER = ...
    """
    Indicates that the numeric string represents a binary value. Valid binary values include the numeric digits 0 and 1.
    Strings that are parsed using this style do not employ a prefix; "0b" cannot be used. A string that is parsed with
    the AllowBinarySpecifier style will always be interpreted as a binary value. The only flags that can
    be combined with AllowBinarySpecifier are AllowLeadingWhite and AllowTrailingWhite.
    The NumberStyles enumeration includes a composite style, BinaryNumber, that consists of
    these three flags.
    """

    INTEGER = ...

    HEX_NUMBER = ...

    BINARY_NUMBER = ...
    """Indicates that the AllowLeadingWhite, AllowTrailingWhite, and AllowBinarySpecifier styles are used. This is a composite number style."""

    NUMBER = ...

    FLOAT = ...

    CURRENCY = ...

    ANY = ...


class DateTimeStyles(Enum):
    """Defines the formatting options that customize string parsing for some date and time parsing methods."""

    NONE = ...

    ALLOW_LEADING_WHITE = ...

    ALLOW_TRAILING_WHITE = ...

    ALLOW_INNER_WHITE = ...

    ALLOW_WHITE_SPACES = ...

    NO_CURRENT_DATE_DEFAULT = ...

    ADJUST_TO_UNIVERSAL = ...

    ASSUME_LOCAL = ...

    ASSUME_UNIVERSAL = ...

    ROUNDTRIP_KIND = ...


class TaiwanCalendar(System.Globalization.Calendar):
    """
    Taiwan calendar is based on the Gregorian calendar.  And the year is an offset to Gregorian calendar.
    That is,
         Taiwan year = Gregorian year - 1911.  So 1912/01/01 A.D. is Taiwan 1/01/01
    """

    @property
    def min_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def max_supported_date_time(self) -> datetime.datetime:
        ...

    @property
    def algorithm_type(self) -> System.Globalization.CalendarAlgorithmType:
        ...

    @property
    def eras(self) -> typing.List[int]:
        ...

    @property
    def two_digit_year_max(self) -> int:
        ...

    @property.setter
    def two_digit_year_max(self, value: int) -> None:
        ...

    def __init__(self) -> None:
        ...

    def add_months(self, time: typing.Union[datetime.datetime, datetime.date], months: int) -> datetime.datetime:
        ...

    def add_years(self, time: typing.Union[datetime.datetime, datetime.date], years: int) -> datetime.datetime:
        ...

    def get_day_of_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_day_of_week(self, time: typing.Union[datetime.datetime, datetime.date]) -> System.DayOfWeek:
        ...

    def get_day_of_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_days_in_month(self, year: int, month: int, era: int) -> int:
        ...

    def get_days_in_year(self, year: int, era: int) -> int:
        ...

    def get_era(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_leap_month(self, year: int, era: int) -> int:
        ...

    def get_month(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def get_months_in_year(self, year: int, era: int) -> int:
        ...

    def get_week_of_year(self, time: typing.Union[datetime.datetime, datetime.date], rule: System.Globalization.CalendarWeekRule, first_day_of_week: System.DayOfWeek) -> int:
        ...

    def get_year(self, time: typing.Union[datetime.datetime, datetime.date]) -> int:
        ...

    def is_leap_day(self, year: int, month: int, day: int, era: int) -> bool:
        ...

    def is_leap_month(self, year: int, month: int, era: int) -> bool:
        ...

    def is_leap_year(self, year: int, era: int) -> bool:
        ...

    def to_date_time(self, year: int, month: int, day: int, hour: int, minute: int, second: int, millisecond: int, era: int) -> datetime.datetime:
        ...

    def to_four_digit_year(self, year: int) -> int:
        """
        For Taiwan calendar, four digit year is not used.
        Therefore, for any two digit number, we just return the original number.
        """
        ...





from typing import overload
from enum import Enum
import datetime
import typing

import QuantConnect
import QuantConnect.Algorithm.Framework.Alphas
import QuantConnect.Algorithm.Framework.Alphas.Analysis
import QuantConnect.AlgorithmFactory.Python.Wrappers
import QuantConnect.Benchmarks
import QuantConnect.Brokerages
import QuantConnect.Commands
import QuantConnect.Data
import QuantConnect.Data.Market
import QuantConnect.Data.UniverseSelection
import QuantConnect.Interfaces
import QuantConnect.Notifications
import QuantConnect.Orders
import QuantConnect.Python
import QuantConnect.Scheduling
import QuantConnect.Securities
import QuantConnect.Securities.Future
import QuantConnect.Securities.Option
import QuantConnect.Statistics
import QuantConnect.Storage
import System
import System.Collections.Concurrent
import System.Collections.Generic

QuantConnect_AlgorithmFactory_Python_Wrappers__EventContainer_Callable = typing.TypeVar("QuantConnect_AlgorithmFactory_Python_Wrappers__EventContainer_Callable")
QuantConnect_AlgorithmFactory_Python_Wrappers__EventContainer_ReturnType = typing.TypeVar("QuantConnect_AlgorithmFactory_Python_Wrappers__EventContainer_ReturnType")


class AlgorithmPythonWrapper(QuantConnect.Python.BasePythonWrapper[QuantConnect.Interfaces.IAlgorithm], QuantConnect.Interfaces.IAlgorithm):
    """Creates and wraps the algorithm written in python."""

    @property
    def is_on_end_of_day_implemented(self) -> bool:
        """True if the underlying python algorithm implements "OnEndOfDay\""""
        ...

    @property
    def is_on_end_of_day_symbol_implemented(self) -> bool:
        """True if the underlying python algorithm implements "OnEndOfDay(symbol)\""""
        ...

    @property
    def algorithm_id(self) -> str:
        """AlgorithmId for the backtest"""
        ...

    @property
    def benchmark(self) -> QuantConnect.Benchmarks.IBenchmark:
        """
        Gets the function used to define the benchmark. This function will return
        the value of the benchmark at a requested date/time
        """
        ...

    @property
    def brokerage_message_handler(self) -> QuantConnect.Brokerages.IBrokerageMessageHandler:
        """
        Gets the brokerage message handler used to decide what to do
        with each message sent from the brokerage
        """
        ...

    @property.setter
    def brokerage_message_handler(self, value: QuantConnect.Brokerages.IBrokerageMessageHandler) -> None:
        ...

    @property
    def brokerage_model(self) -> QuantConnect.Brokerages.IBrokerageModel:
        """Gets the brokerage model used to emulate a real brokerage"""
        ...

    @property
    def brokerage_name(self) -> QuantConnect.Brokerages.BrokerageName:
        """Gets the brokerage name."""
        ...

    @property
    def risk_free_interest_rate_model(self) -> QuantConnect.Data.IRiskFreeInterestRateModel:
        """Gets the risk free interest rate model used to get the interest rates"""
        ...

    @property
    def debug_messages(self) -> System.Collections.Concurrent.ConcurrentQueue[str]:
        """Debug messages from the strategy:"""
        ...

    @property
    def end_date(self) -> datetime.datetime:
        """Get Requested Backtest End Date"""
        ...

    @property
    def error_messages(self) -> System.Collections.Concurrent.ConcurrentQueue[str]:
        """Error messages from the strategy:"""
        ...

    @property
    def history_provider(self) -> QuantConnect.Interfaces.IHistoryProvider:
        """Gets or sets the history provider for the algorithm"""
        ...

    @property.setter
    def history_provider(self, value: QuantConnect.Interfaces.IHistoryProvider) -> None:
        ...

    @property
    def is_warming_up(self) -> bool:
        """Gets whether or not this algorithm is still warming up"""
        ...

    @property
    def live_mode(self) -> bool:
        """Algorithm is running on a live server."""
        ...

    @property
    def algorithm_mode(self) -> QuantConnect.AlgorithmMode:
        """Algorithm running mode."""
        ...

    @property
    def deployment_target(self) -> QuantConnect.DeploymentTarget:
        """Deployment target, either local or cloud."""
        ...

    @property
    def log_messages(self) -> System.Collections.Concurrent.ConcurrentQueue[str]:
        """Log messages from the strategy:"""
        ...

    @property
    def name(self) -> str:
        """Public name for the algorithm."""
        ...

    @property.setter
    def name(self, value: str) -> None:
        ...

    @property
    def tags(self) -> System.Collections.Generic.HashSet[str]:
        """A list of tags associated with the algorithm or the backtest, useful for categorization"""
        ...

    @property.setter
    def tags(self, value: System.Collections.Generic.HashSet[str]) -> None:
        ...

    @property
    def name_updated(self) -> _EventContainer[typing.Callable[[QuantConnect.Interfaces.IAlgorithm, str], None], None]:
        """Event fired algorithm's name is changed"""
        ...

    @property
    def tags_updated(self) -> _EventContainer[typing.Callable[[QuantConnect.Interfaces.IAlgorithm, System.Collections.Generic.HashSet[str]], None], None]:
        """Event fired when the tag collection is updated"""
        ...

    @property
    def notify(self) -> QuantConnect.Notifications.NotificationManager:
        """Notification manager for storing and processing live event messages"""
        ...

    @property
    def portfolio(self) -> QuantConnect.Securities.SecurityPortfolioManager:
        """
        Security portfolio management class provides wrapper and helper methods for the Security.Holdings class such as
        IsLong, IsShort, TotalProfit
        """
        ...

    @property
    def run_time_error(self) -> System.Exception:
        """Gets the run time error from the algorithm, or null if none was encountered."""
        ...

    @property.setter
    def run_time_error(self, value: System.Exception) -> None:
        ...

    @property
    def runtime_statistics(self) -> System.Collections.Concurrent.ConcurrentDictionary[str, str]:
        """Customizable dynamic statistics displayed during live trading:"""
        ...

    @property
    def schedule(self) -> QuantConnect.Scheduling.ScheduleManager:
        """Gets schedule manager for adding/removing scheduled events"""
        ...

    @property
    def securities(self) -> QuantConnect.Securities.SecurityManager:
        """
        Security object collection class stores an array of objects representing representing each security/asset
        we have a subscription for.
        """
        ...

    @property
    def security_initializer(self) -> QuantConnect.Securities.ISecurityInitializer:
        """Gets an instance that is to be used to initialize newly created securities."""
        ...

    @property
    def trade_builder(self) -> QuantConnect.Interfaces.ITradeBuilder:
        """Gets the Trade Builder to generate trades from executions"""
        ...

    @property
    def settings(self) -> QuantConnect.Interfaces.IAlgorithmSettings:
        """Gets the user settings for the algorithm"""
        ...

    @property
    def option_chain_provider(self) -> QuantConnect.Interfaces.IOptionChainProvider:
        """Gets the option chain provider, used to get the list of option contracts for an underlying symbol"""
        ...

    @property
    def future_chain_provider(self) -> QuantConnect.Interfaces.IFutureChainProvider:
        """Gets the future chain provider, used to get the list of future contracts for an underlying symbol"""
        ...

    @property
    def object_store(self) -> QuantConnect.Storage.ObjectStore:
        """Gets the object store, used for persistence"""
        ...

    @property
    def current_slice(self) -> QuantConnect.Data.Slice:
        """Returns the current Slice object"""
        ...

    @property
    def start_date(self) -> datetime.datetime:
        """Algorithm start date for backtesting, set by the SetStartDate methods."""
        ...

    @property
    def status(self) -> QuantConnect.AlgorithmStatus:
        """Gets or sets the current status of the algorithm"""
        ...

    @property.setter
    def status(self, value: QuantConnect.AlgorithmStatus) -> None:
        ...

    @property
    def insights_generated(self) -> _EventContainer[typing.Callable[[QuantConnect.Interfaces.IAlgorithm, QuantConnect.Algorithm.Framework.Alphas.GeneratedInsightsCollection], None], None]:
        """Event fired when an algorithm generates a insight"""
        ...

    @property
    def time_keeper(self) -> QuantConnect.Interfaces.ITimeKeeper:
        """Gets the time keeper instance"""
        ...

    @property
    def subscription_manager(self) -> QuantConnect.Data.SubscriptionManager:
        """
        Data subscription manager controls the information and subscriptions the algorithms recieves.
        Subscription configurations can be added through the Subscription Manager.
        """
        ...

    @property
    def project_id(self) -> int:
        """The project id associated with this algorithm if any"""
        ...

    @property.setter
    def project_id(self, value: int) -> None:
        ...

    @property
    def time(self) -> datetime.datetime:
        """Current date/time in the algorithm's local time zone"""
        ...

    @property
    def time_zone(self) -> typing.Any:
        """Gets the time zone of the algorithm"""
        ...

    @property
    def transactions(self) -> QuantConnect.Securities.SecurityTransactionManager:
        """Security transaction manager class controls the store and processing of orders."""
        ...

    @property
    def universe_manager(self) -> QuantConnect.Securities.UniverseManager:
        """Gets the collection of universes for the algorithm"""
        ...

    @property
    def universe_settings(self) -> QuantConnect.Data.UniverseSelection.UniverseSettings:
        """Gets the subscription settings to be used when adding securities via universe selection"""
        ...

    @property
    def utc_time(self) -> datetime.datetime:
        """Current date/time in UTC."""
        ...

    @property
    def account_currency(self) -> str:
        """Gets the account currency"""
        ...

    @property
    def insights(self) -> QuantConnect.Algorithm.Framework.Alphas.Analysis.InsightManager:
        """Gets the insight manager"""
        ...

    @property
    def statistics(self) -> QuantConnect.Statistics.StatisticsResults:
        """The current statistics for the running algorithm."""
        ...

    def __init__(self, moduleName: str) -> None:
        """
        AlgorithmPythonWrapper constructor.
        Creates and wraps the algorithm written in python.
        
        :param moduleName: Name of the module that can be found in the PYTHONPATH
        """
        ...

    def add_chart(self, chart: QuantConnect.Chart) -> None:
        """
        Add a Chart object to algorithm collection
        
        :param chart: Chart object to add to collection.
        """
        ...

    def add_future_contract(self, symbol: typing.Union[QuantConnect.Symbol, str], resolution: typing.Optional[QuantConnect.Resolution] = None, fill_forward: bool = True, leverage: float = 0, extended_market_hours: bool = False) -> QuantConnect.Securities.Future.Future:
        """
        Creates and adds a new single Future contract to the algorithm
        
        :param symbol: The futures contract symbol
        :param resolution: The Resolution of market data, Tick, Second, Minute, Hour, or Daily. Default is Resolution.Minute
        :param fill_forward: If true, returns the last available data even if none in that timeslice. Default is true
        :param leverage: The requested leverage for this equity. Default is set by SecurityInitializer
        :param extended_market_hours: Use extended market hours data
        :returns: The new Future security.
        """
        ...

    def add_option_contract(self, symbol: typing.Union[QuantConnect.Symbol, str], resolution: typing.Optional[QuantConnect.Resolution] = None, fill_forward: bool = True, leverage: float = 0, extended_market_hours: bool = False) -> QuantConnect.Securities.Option.Option:
        """
        Creates and adds a new single Option contract to the algorithm
        
        :param symbol: The option contract symbol
        :param resolution: The Resolution of market data, Tick, Second, Minute, Hour, or Daily. Default is Resolution.Minute
        :param fill_forward: If true, returns the last available data even if none in that timeslice. Default is true
        :param leverage: The requested leverage for this equity. Default is set by SecurityInitializer
        :param extended_market_hours: Use extended market hours data
        :returns: The new Option security.
        """
        ...

    @overload
    def add_security(self, security_type: QuantConnect.SecurityType, symbol: str, resolution: typing.Optional[QuantConnect.Resolution], market: str, fill_forward: bool, leverage: float, extended_market_hours: bool, data_mapping_mode: typing.Optional[QuantConnect.DataMappingMode] = None, data_normalization_mode: typing.Optional[QuantConnect.DataNormalizationMode] = None) -> QuantConnect.Securities.Security:
        """
        Set a required SecurityType-symbol and resolution for algorithm
        
        :param security_type: SecurityType Enum: Equity, Commodity, FOREX or Future
        :param symbol: Symbol Representation of the MarketType, e.g. AAPL
        :param resolution: The Resolution of market data, Tick, Second, Minute, Hour, or Daily.
        :param market: The market the requested security belongs to, such as 'usa' or 'fxcm'
        :param fill_forward: If true, returns the last available data even if none in that timeslice.
        :param leverage: leverage for this security
        :param extended_market_hours: Use extended market hours data
        :param data_mapping_mode: The contract mapping mode to use for the security
        :param data_normalization_mode: The price scaling mode to use for the security
        """
        ...

    @overload
    def add_security(self, symbol: typing.Union[QuantConnect.Symbol, str], resolution: typing.Optional[QuantConnect.Resolution] = None, fill_forward: bool = True, leverage: float = ..., extended_market_hours: bool = False, data_mapping_mode: typing.Optional[QuantConnect.DataMappingMode] = None, data_normalization_mode: typing.Optional[QuantConnect.DataNormalizationMode] = None, contract_depth_offset: int = 0) -> QuantConnect.Securities.Security:
        """
        Set a required SecurityType-symbol and resolution for algorithm
        
        :param symbol: The security Symbol
        :param resolution: Resolution of the MarketType required: MarketData, Second or Minute
        :param fill_forward: If true, returns the last available data even if none in that timeslice.
        :param leverage: leverage for this security
        :param extended_market_hours: Use extended market hours data
        :param data_mapping_mode: The contract mapping mode to use for the security
        :param data_normalization_mode: The price scaling mode to use for the security
        :param contract_depth_offset: The continuous contract desired offset from the current front month. For example, 0 (default) will use the front month, 1 will use the back month contract
        :returns: The new Security that was added to the algorithm.
        """
        ...

    def add_tag(self, tag: str) -> None:
        """
        Adds a tag to the algorithm
        
        :param tag: The tag to add
        """
        ...

    def debug(self, message: str) -> None:
        """
        Send debug message
        
        :param message: String message
        """
        ...

    def error(self, message: str) -> None:
        """
        Send an error message for the algorithm
        
        :param message: String message
        """
        ...

    def get_chart_updates(self, clear_chart_data: bool = False) -> System.Collections.Generic.IEnumerable[QuantConnect.Chart]:
        """
        Get the chart updates since the last request:
        
        :returns: List of Chart Updates.
        """
        ...

    def get_last_known_price(self, security: QuantConnect.Securities.Security) -> QuantConnect.Data.BaseData:
        """
        Get the last known price using the history provider.
        Useful for seeding securities with the correct price
        
        :param security: Security object for which to retrieve historical data
        :returns: A single BaseData object with the last known price.
        """
        ...

    def get_locked(self) -> bool:
        """Gets whether or not this algorithm has been locked and fully initialized"""
        ...

    @overload
    def get_parameter(self, name: str, default_value: str = None) -> str:
        """
        Gets the parameter with the specified name. If a parameter with the specified name does not exist,
        the given default value is returned if any, else null
        
        :param name: The name of the parameter to get
        :param default_value: The default value to return
        :returns: The value of the specified parameter, or default_value if not found or null if there's no default value.
        """
        ...

    @overload
    def get_parameter(self, name: str, default_value: int) -> int:
        """
        Gets the parameter with the specified name parsed as an integer. If a parameter with the specified name does not exist,
        or the conversion is not possible, the given default value is returned
        
        :param name: The name of the parameter to get
        :param default_value: The default value to return
        :returns: The value of the specified parameter, or default_value if not found or null if there's no default value.
        """
        ...

    @overload
    def get_parameter(self, name: str, default_value: float) -> float:
        """
        Gets the parameter with the specified name parsed as a double. If a parameter with the specified name does not exist,
        or the conversion is not possible, the given default value is returned
        
        :param name: The name of the parameter to get
        :param default_value: The default value to return
        :returns: The value of the specified parameter, or default_value if not found or null if there's no default value.
        """
        ...

    @overload
    def get_parameter(self, name: str, default_value: float) -> float:
        """
        Gets the parameter with the specified name parsed as a decimal. If a parameter with the specified name does not exist,
        or the conversion is not possible, the given default value is returned
        
        :param name: The name of the parameter to get
        :param default_value: The default value to return
        :returns: The value of the specified parameter, or default_value if not found or null if there's no default value.
        """
        ...

    def get_parameters(self) -> System.Collections.Generic.IReadOnlyDictionary[str, str]:
        """Gets a read-only dictionary with all current parameters"""
        ...

    def initialize(self) -> None:
        """Initialise the Algorithm and Prepare Required Data:"""
        ...

    def liquidate(self, symbol: typing.Union[QuantConnect.Symbol, str] = None, asynchronous: bool = False, tag: str = "Liquidated", order_properties: QuantConnect.Interfaces.IOrderProperties = None) -> System.Collections.Generic.List[QuantConnect.Orders.OrderTicket]:
        """
        Liquidate your portfolio holdings
        
        :param symbol: Specific asset to liquidate, defaults to all
        :param asynchronous: Flag to indicate if the symbols should be liquidated asynchronously
        :param tag: Custom tag to know who is calling this
        :param order_properties: Order properties to use
        """
        ...

    def log(self, message: str) -> None:
        """
        Save entry to the Log
        
        :param message: String message
        """
        ...

    def on_assignment_order_event(self, assignment_event: QuantConnect.Orders.OrderEvent) -> None:
        """
        Option assignment event handler. On an option assignment event for short legs the resulting information is passed to this method.
        
        :param assignment_event: Option exercise event details containing details of the assignment
        """
        ...

    def on_brokerage_disconnect(self) -> None:
        """Brokerage disconnected event handler. This method is called when the brokerage connection is lost."""
        ...

    def on_brokerage_message(self, message_event: QuantConnect.Brokerages.BrokerageMessageEvent) -> None:
        """Brokerage message event handler. This method is called for all types of brokerage messages."""
        ...

    def on_brokerage_reconnect(self) -> None:
        """Brokerage reconnected event handler. This method is called when the brokerage connection is restored after a disconnection."""
        ...

    def on_command(self, data: typing.Any) -> typing.Optional[bool]:
        """
        Generic untyped command call handler
        
        :param data: The associated data
        :returns: True if success, false otherwise. Returning null will disable command feedback.
        """
        ...

    def on_data(self, slice: QuantConnect.Data.Slice) -> None:
        """
        v3.0 Handler for all data types
        
        :param slice: The current slice of data
        """
        ...

    def on_delistings(self, delistings: QuantConnect.Data.Market.Delistings) -> None:
        """
        Event handler to be called when there's been a delistings event
        
        :param delistings: The current time slice delistings
        """
        ...

    def on_dividends(self, dividends: QuantConnect.Data.Market.Dividends) -> None:
        """
        Event handler to be called when there's been a dividend event
        
        :param dividends: The current time slice dividends
        """
        ...

    def on_end_of_algorithm(self) -> None:
        """Call this event at the end of the algorithm running."""
        ...

    @overload
    def on_end_of_day(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> None:
        """
        End of a trading day event handler. This method is called at the end of the algorithm day (or multiple times if trading multiple assets).
        
        :param symbol: Asset symbol for this end of day event. Forex and equities have different closing hours.
        """
        ...

    @overload
    def on_end_of_day(self) -> None:
        """
        End of a trading day event handler. This method is called at the end of the algorithm day (or multiple times if trading multiple assets).
        
        This method is deprecated. Please use this overload: OnEndOfDay(Symbol symbol)
        """
        ...

    def on_end_of_time_step(self) -> None:
        """
        Invoked at the end of every time step. This allows the algorithm
        to process events before advancing to the next time step.
        """
        ...

    def on_framework_data(self, slice: QuantConnect.Data.Slice) -> None:
        """
        Used to send data updates to algorithm framework models
        
        :param slice: The current data slice
        """
        ...

    def on_framework_securities_changed(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """
        Used to send security changes to algorithm framework models
        
        :param changes: Security additions/removals for this time step
        """
        ...

    def on_margin_call(self, requests: System.Collections.Generic.List[QuantConnect.Orders.SubmitOrderRequest]) -> None:
        """
        Margin call event handler. This method is called right before the margin call orders are placed in the market.
        
        :param requests: The orders to be executed to bring this algorithm within margin limits
        """
        ...

    def on_margin_call_warning(self) -> None:
        """Margin call warning event handler. This method is called when Portfolio.MarginRemaining is under 5% of your Portfolio.TotalPortfolioValue"""
        ...

    def on_order_event(self, new_event: QuantConnect.Orders.OrderEvent) -> None:
        """
        EXPERTS ONLY:: [-!-Async Code-!-]
        New order event handler: on order status changes (filled, partially filled, cancelled etc).
        
        :param new_event: Event information
        """
        ...

    def on_securities_changed(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """
        Event fired each time the we add/remove securities from the data feed
        
        :param changes: Security additions/removals for this time step
        """
        ...

    def on_splits(self, splits: QuantConnect.Data.Market.Splits) -> None:
        """
        Event handler to be called when there's been a split event
        
        :param splits: The current time slice splits
        """
        ...

    def on_symbol_changed_events(self, symbols_changed: QuantConnect.Data.Market.SymbolChangedEvents) -> None:
        """
        Event handler to be called when there's been a symbol changed event
        
        :param symbols_changed: The current time slice symbol changed events
        """
        ...

    def on_warmup_finished(self) -> None:
        """Called when the algorithm has completed initialization and warm up."""
        ...

    def post_initialize(self) -> None:
        """
        Called by setup handlers after Initialize and allows the algorithm a chance to organize
        the data gather in the Initialize method
        """
        ...

    def remove_security(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> bool:
        """
        Removes the security with the specified symbol. This will cancel all
        open orders and then liquidate any existing holdings
        
        :param symbol: The symbol of the security to be removed
        """
        ...

    def run_command(self, command: QuantConnect.Commands.CallbackCommand) -> QuantConnect.Commands.CommandResultPacket:
        """
        Run a callback command instance
        
        :param command: The callback command instance
        :returns: The command result.
        """
        ...

    def set_account_currency(self, account_currency: str, starting_cash: typing.Optional[float] = None) -> None:
        """
        Sets the account currency cash symbol this algorithm is to manage, as well
        as the starting cash in this currency if given
        
        :param account_currency: The account currency cash symbol to set
        :param starting_cash: The account currency starting cash to set
        """
        ...

    def set_algorithm_id(self, algorithm_id: str) -> None:
        """
        Set the algorithm Id for this backtest or live run. This can be used to identify the order and equity records.
        
        :param algorithm_id: unique 32 character identifier for backtest or live server
        """
        ...

    def set_algorithm_mode(self, algorithm_mode: QuantConnect.AlgorithmMode) -> None:
        """
        Sets the algorithm running mode
        
        :param algorithm_mode: Algorithm mode
        """
        ...

    def set_api(self, api: QuantConnect.Interfaces.IApi) -> None:
        """
        Provide the API for the algorithm.
        
        :param api: Initiated API
        """
        ...

    def set_available_data_types(self, available_data_types: System.Collections.Generic.Dictionary[QuantConnect.SecurityType, System.Collections.Generic.List[QuantConnect.TickType]]) -> None:
        """
        Set the available TickType supported by each SecurityType in SecurityManager
        
        :param available_data_types: >The different TickType each Security supports
        """
        ...

    def set_brokerage_message_handler(self, handler: QuantConnect.Brokerages.IBrokerageMessageHandler) -> None:
        """
        Sets the implementation used to handle messages from the brokerage.
        The default implementation will forward messages to debug or error
        and when a BrokerageMessageType.Error occurs, the algorithm
        is stopped.
        
        :param handler: The message handler to use
        """
        ...

    def set_brokerage_model(self, brokerage_model: QuantConnect.Brokerages.IBrokerageModel) -> None:
        """
        Sets the brokerage model used to resolve transaction models, settlement models,
        and brokerage specified ordering behaviors.
        
        :param brokerage_model: The brokerage model used to emulate the real brokerage
        """
        ...

    @overload
    def set_cash(self, starting_cash: float) -> None:
        """
        Set the starting capital for the strategy
        
        :param starting_cash: decimal starting capital, default $100,000
        """
        ...

    @overload
    def set_cash(self, symbol: str, starting_cash: float, conversion_rate: float = 0) -> None:
        """
        Set the cash for the specified symbol
        
        :param symbol: The cash symbol to set
        :param starting_cash: Decimal cash value of portfolio
        :param conversion_rate: The current conversion rate for the
        """
        ...

    def set_current_slice(self, slice: QuantConnect.Data.Slice) -> None:
        """
        Sets the current slice
        
        :param slice: The Slice object
        """
        ...

    def set_date_time(self, time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """Set the DateTime Frontier: This is the master time and is"""
        ...

    def set_deployment_target(self, deployment_target: QuantConnect.DeploymentTarget) -> None:
        """
        Sets the algorithm deployment target
        
        :param deployment_target: Deployment target
        """
        ...

    def set_end_date(self, end: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Set the end date for a backtest.
        
        :param end: Datetime value for end date
        """
        ...

    def set_finished_warming_up(self) -> None:
        """Sets IsWarmingUp to false to indicate this algorithm has finished its warm up"""
        ...

    def set_future_chain_provider(self, future_chain_provider: QuantConnect.Interfaces.IFutureChainProvider) -> None:
        """
        Sets the future chain provider, used to get the list of future contracts for an underlying symbol
        
        :param future_chain_provider: The future chain provider
        """
        ...

    def set_history_provider(self, history_provider: QuantConnect.Interfaces.IHistoryProvider) -> None:
        """
        Set the historical data provider
        
        :param history_provider: Historical data provider
        """
        ...

    def set_live_mode(self, live: bool) -> None:
        """
        Set live mode state of the algorithm run: Public setter for the algorithm property LiveMode.
        
        :param live: Bool live mode flag
        """
        ...

    def set_locked(self) -> None:
        """Set the algorithm as initialized and locked. No more cash or security changes."""
        ...

    def set_maximum_orders(self, max: int) -> None:
        """
        Set the maximum number of orders the algorithm is allowed to process.
        
        :param max: Maximum order count int
        """
        ...

    def set_name(self, name: str) -> None:
        """
        Sets name to the currently running backtest
        
        :param name: The name for the backtest
        """
        ...

    def set_object_store(self, object_store: QuantConnect.Interfaces.IObjectStore) -> None:
        """
        Sets the object store
        
        :param object_store: The object store
        """
        ...

    def set_option_chain_provider(self, option_chain_provider: QuantConnect.Interfaces.IOptionChainProvider) -> None:
        """
        Sets the option chain provider, used to get the list of option contracts for an underlying symbol
        
        :param option_chain_provider: The option chain provider
        """
        ...

    def set_parameters(self, parameters: System.Collections.Generic.Dictionary[str, str]) -> None:
        """
        Sets the parameters from the dictionary
        
        :param parameters: Dictionary containing the parameter names to values
        """
        ...

    def set_run_time_error(self, exception: System.Exception) -> None:
        """
        Set the runtime error
        
        :param exception: Represents error that occur during execution
        """
        ...

    def set_start_date(self, start: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Set the start date for the backtest
        
        :param start: Datetime Start date for backtest
        """
        ...

    def set_statistics_service(self, statistics_service: QuantConnect.Statistics.IStatisticsService) -> None:
        """
        Sets the statistics service instance to be used by the algorithm
        
        :param statistics_service: The statistics service instance
        """
        ...

    def set_status(self, status: QuantConnect.AlgorithmStatus) -> None:
        """
        Set the state of a live deployment
        
        :param status: Live deployment status
        """
        ...

    def set_tags(self, tags: System.Collections.Generic.HashSet[str]) -> None:
        """
        Sets the tags for the algorithm
        
        :param tags: The tags
        """
        ...

    def shortable(self, symbol: typing.Union[QuantConnect.Symbol, str], short_quantity: float, update_order_id: typing.Optional[int] = None) -> bool:
        """
        Determines if the Symbol is shortable at the brokerage
        
        :param symbol: Symbol to check if shortable
        :param short_quantity: Order's quantity to check if it is currently shortable, taking into account current holdings and open orders
        :param update_order_id: Optionally the id of the order being updated. When updating an order we want to ignore it's submitted short quantity and use the new provided quantity to determine if we can perform the update
        :returns: True if the symbol can be shorted by the requested quantity.
        """
        ...

    def shortable_quantity(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> int:
        """
        Gets the quantity shortable for the given asset
        
        :returns: Quantity shortable for the given asset. Zero if not shortable, or a number greater than zero if shortable.
        """
        ...

    def submit_order_request(self, request: QuantConnect.Orders.SubmitOrderRequest) -> QuantConnect.Orders.OrderTicket:
        """
        Will submit an order request to the algorithm
        
        :param request: The request to submit
        :returns: The order ticket.
        """
        ...

    def symbol(self, ticker: str) -> QuantConnect.Symbol:
        """
        Converts the string 'ticker' symbol into a full Symbol object
        This requires that the string 'ticker' has been added to the algorithm
        
        :param ticker: The ticker symbol. This should be the ticker symbol as it was added to the algorithm
        :returns: The symbol object mapped to the specified ticker.
        """
        ...

    def ticker(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> str:
        """
        For the given symbol will resolve the ticker it used at the current algorithm date
        
        :param symbol: The symbol to get the ticker for
        :returns: The mapped ticker for a symbol.
        """
        ...

    def to_string(self) -> str:
        """Returns a string that represents the current AlgorithmPythonWrapper object."""
        ...


class _EventContainer(typing.Generic[QuantConnect_AlgorithmFactory_Python_Wrappers__EventContainer_Callable, QuantConnect_AlgorithmFactory_Python_Wrappers__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> QuantConnect_AlgorithmFactory_Python_Wrappers__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: QuantConnect_AlgorithmFactory_Python_Wrappers__EventContainer_Callable) -> None:
        """Registers an event handler."""
        ...

    def __isub__(self, item: QuantConnect_AlgorithmFactory_Python_Wrappers__EventContainer_Callable) -> None:
        """Unregisters an event handler."""
        ...





from typing import overload
from enum import Enum
import typing

import System
import System.Runtime.ExceptionServices


class HandleProcessCorruptedStateExceptionsAttribute(System.Attribute):
    """Obsoletions.CorruptedStateRecoveryMessage"""

    def __init__(self) -> None:
        ...


class FirstChanceExceptionEventArgs(System.EventArgs):
    """This class has no documentation."""

    @property
    def exception(self) -> System.Exception:
        ...

    def __init__(self, exception: System.Exception) -> None:
        ...


class ExceptionDispatchInfo(System.Object):
    """This class has no documentation."""

    @property
    def source_exception(self) -> System.Exception:
        ...

    @staticmethod
    def capture(source: System.Exception) -> System.Runtime.ExceptionServices.ExceptionDispatchInfo:
        ...

    @staticmethod
    def set_current_stack_trace(source: System.Exception) -> System.Exception:
        """
        Stores the current stack trace into the specified Exception instance.
        
        :param source: The unthrown Exception instance.
        :returns: The  exception instance.
        """
        ...

    @staticmethod
    def set_remote_stack_trace(source: System.Exception, stack_trace: str) -> System.Exception:
        """
        Stores the provided stack trace into the specified Exception instance.
        
        :param source: The unthrown Exception instance.
        :param stack_trace: The stack trace string to persist within . This is normally acquired from the Exception.StackTrace property from the remote exception instance.
        :returns: The  exception instance.
        """
        ...

    @overload
    def throw(self) -> None:
        ...

    @staticmethod
    @overload
    def throw(source: System.Exception) -> None:
        ...


class ExceptionHandling(System.Object):
    """This class has no documentation."""

    @staticmethod
    def set_unhandled_exception_handler(handler: typing.Callable[[System.Exception], bool]) -> None:
        """Sets a handler for unhandled exceptions."""
        ...





from typing import overload
from enum import Enum
import datetime
import typing

import QuantConnect
import QuantConnect.Algorithm
import QuantConnect.Algorithm.Framework.Selection
import QuantConnect.Algorithm.Selection
import QuantConnect.Data
import QuantConnect.Data.UniverseSelection
import QuantConnect.Securities
import System.Collections.Generic
import System.Collections.Specialized


class OptionChainedUniverseSelectionModel(QuantConnect.Algorithm.Framework.Selection.UniverseSelectionModel):
    """
    This universe selection model will chain to the security changes of a given Universe selection
    output and create a new OptionChainUniverse for each of them
    """

    @overload
    def __init__(self, universe: QuantConnect.Data.UniverseSelection.Universe, optionFilter: typing.Callable[[QuantConnect.Securities.OptionFilterUniverse], QuantConnect.Securities.OptionFilterUniverse], universeSettings: QuantConnect.Data.UniverseSelection.UniverseSettings = None) -> None:
        """
        Creates a new instance of OptionChainedUniverseSelectionModel
        
        :param universe: The universe we want to chain to
        :param optionFilter: The option filter universe to use
        :param universeSettings: Universe settings define attributes of created subscriptions, such as their resolution and the minimum time in universe before they can be removed
        """
        ...

    @overload
    def __init__(self, universe: QuantConnect.Data.UniverseSelection.Universe, optionFilter: typing.Any, universeSettings: QuantConnect.Data.UniverseSelection.UniverseSettings = None) -> None:
        """
        Creates a new instance of OptionChainedUniverseSelectionModel
        
        :param universe: The universe we want to chain to
        :param optionFilter: The python option filter universe to use
        :param universeSettings: Universe settings define attributes of created subscriptions, such as their resolution and the minimum time in universe before they can be removed
        """
        ...

    def create_universes(self, algorithm: QuantConnect.Algorithm.QCAlgorithm) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.UniverseSelection.Universe]:
        """
        Creates the universes for this algorithm. Called once after IAlgorithm.Initialize
        
        :param algorithm: The algorithm instance to create universes for
        :returns: The universes to be used by the algorithm.
        """
        ...

    def get_next_refresh_time_utc(self) -> datetime.datetime:
        """Gets the next time the framework should invoke the `CreateUniverses` method to refresh the set of universes."""
        ...


class OptionContractUniverse(QuantConnect.Data.UniverseSelection.UserDefinedUniverse):
    """This universe will hold single option contracts and their underlying, managing removals and additions"""

    def __init__(self, configuration: QuantConnect.Data.SubscriptionDataConfig, universeSettings: QuantConnect.Data.UniverseSelection.UniverseSettings) -> None:
        """
        Creates a new empty instance
        
        :param configuration: The universe configuration to use
        :param universeSettings: The universe settings to use
        """
        ...

    @staticmethod
    def create_symbol(market: str, security_type: QuantConnect.SecurityType) -> QuantConnect.Symbol:
        """
        Creates a user defined universe symbol
        
        :param market: The market
        :param security_type: The underlying option security type
        :returns: A symbol for user defined universe of the specified security type and market.
        """
        ...

    def on_collection_changed(self, e: System.Collections.Specialized.NotifyCollectionChangedEventArgs) -> None:
        """
        Event invocator for the UserDefinedUniverse.CollectionChanged event
        
        This method is protected.
        
        :param e: The notify collection changed event arguments
        """
        ...

    def select_symbols(self, utc_time: typing.Union[datetime.datetime, datetime.date], data: QuantConnect.Data.UniverseSelection.BaseDataCollection) -> System.Collections.Generic.IEnumerable[QuantConnect.Symbol]:
        """
        Returns the symbols defined by the user for this universe
        
        :param utc_time: The current utc time
        :param data: The symbols to remain in the universe
        :returns: The data that passes the filter.
        """
        ...


